#!/usr/bin/env python

#
##########################
#                        #
#       Test Cases       #
#                        #
##########################
#
# [test_ebs_extended_test_suite]
#       Full suite of ebs related tests
#        Test Summary: 
#
#        -create a volume (do this first)
#        -run an instance (do this second, if this fails at least we know we could create a vol)
#        
#        Usage Tests: 
#        -negative -attempt to attach a volume to an instance in a separate cluster. 
#        -attach a single volume to an instance in the zones given, write random data and calc md5 of volumes
#        -detach the volume
#        -create multiple consecutive snapshots from an existin euvolume with calculated md5 sum
#        -create a single volume from each new snapshot
#        -attach each new volume created from the snapshots to an instance 
#        -calculate the md5sum of the new volumes vs the original verify they are the same
#        -remove all test volumes and snapshots used in this test
#        -terminate each test instance
#
#        Cleanup:
#        --remove all volumes, instance, and snapshots created during this test
#
#    @author: clarkmatthew

import unittest
from eutester.eutestcase import EutesterTestCase
from eutester.eutestcase import EutesterTestResult
from ebstestsuite import EbsTestSuite
import argparse
import os



if __name__ == "__main__":
    ## If given command line arguments, use them as test names to launch

    #create a eutester testcase object...
    testcase= EutesterTestCase(name='ebs_extended_test')

    #add/remove arguements to the default cli and config file argument list, see testcase class for all default cli args
    testcase.setup_parser(description="Attempts to test and provide info on focused areas related to Eucalyptus EBS related functionality.", 
                          testlist=False)
    testcase.parser.add_argument('--snap_count', type=int, help='Number of snapshots to create per zone',default=5)
    testcase.parser.add_argument('--snap_delay', type=int, help='Delay in seconds between each snapshot created',default=0)
    testcase.parser.add_argument('--snap_progress', type=int, help='Number of 10 second polls to allow without increase in snapshot progress',default=60)
    testcase.parser.add_argument('--timepergig', type=int, help='Time allowed per gig size of volume during volume creation',default=500)
    testcase.parser.add_argument('--snap_attached', dest='snap_attached', action='store_true', default=True)
    testcase.parser.add_argument('--delete_to', type=int, help="Timeout for volume deletion",  default=180)
    testcase.parser.add_argument('--root_device_type', help="Type of instance to run, 'ebs' or 'instance-store'",  default='instance-store')

    #Get arguments passed in through cli and/or config file(s), these will be stored in testcase.args
    testcase.get_args()

    #create the EbsTestSuite object, use 'do_with_args' to auto populate the object's init with testcase.args
    ebstestsuite= testcase.do_with_args(EbsTestSuite)

    #register a test cleanup method with the testcase...
    testcase.clean_method = ebstestsuite.clean_created_resources

    #add test methods and their arguments to be run to a list...
    testlist = ebstestsuite.test_consecutive_concurrent(run=False, 
                                                    count=int(testcase.args.snap_count),
                                                    delay=testcase.args.snap_delay,
                                                    tpg=testcase.args.timepergig,
                                                    snap_attached = testcase.args.snap_attached,
                                                    delete_to = testcase.args.delete_to,
                                                    poll_progress=testcase.args.snap_progress)

    #finally, execute the list of test methods from the test case wrapper...
    ret = testcase.run_test_case_list(testlist)
    print "ebs_extended_test exiting:("+str(ret)+")"
    exit(ret)

    
  



# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com
"""
Created on Mar 7, 2012
@author: clarkmatthew

simple class to establish an ssh session
example usage:
    import sshconnection
    ssh = SshConnection( '192.168.1.1', keypath='/home/testuser/keyfile.pem')

    #use sys() to get either a list of output lines or a single string buffer depending on listformat flag
    output = ssh.sys('ls /dev/sd*',timeout=10)
    print output[0]
    print ssh.lastcmd+" exited with code: "+str(ssh.lastexitcode)

    #...or use cmd to get a dict of output, exitstatus, and elapsed time to execute...
    out = ssh.cmd('ping 192.168.1.2 -c 1 -W 5')

     print out['cmd']+" exited with status:"+out['status']+", elapsed time:"+out['elapsed']
     print out['output']

example with proxy:
    import sshconnection

    instance_private_ip = '10.1.1.5'
    instance_user = 'root'
    instance_keypath = '/tmp/instancekey.pem'

    proxy_ip = '192.168.1.2'
    proxy_username = 'testuser'
    #proxy_password = 'foo'
    proxy_keypath = '/home/testuser/keyfile.pem'


    example output from ipython:

    In[4]: instance_ssh = SshConnection( instance_private_ip, username=instance_user, keypath=instance_keypath,
            proxy=proxy_ip, proxy_username = proxy_username, proxy_keypath = proxy_keypath, debug_connect = True)
    ssh_connect args:
    hostname:10.1.1.5
    username:root
    password:None
    keypath:/tmp/instancekey.pem
    timeout:60
    retry:1
    IPV6 DNS lookup disabled, do IPV4 resolution and pass IP to connect()
    10.1.1.5, is already an ip, dont do host lookup...
    192.168.1.2, is already an ip, dont do host lookup...
    SSH connection attempt(1 of 2), host:'root@10.1.1.5', using ipv4:10.1.1.5, thru proxy:'192.168.1.2'
    Using Keypath:/tmp/instancekey.pem
    SSH - Connected to 10.1.1.5 via proxy host:192.168.1.2:22

    In [5]: instance_ssh.sys('hostname')
    Out[5]: ['euca_10_1_1_5.eucalyptus_cloud.com']

"""


import copy
import os
import paramiko
import re
import select
import socket
import time
import types
import sys
import termios
import tty
import eucaops




class SshCbReturn():
    def __init__(self, stop=False, statuscode=-1, settimer=0, buf=None, sendstring=None, nextargs=None, nextcb=None, removecb=False):
        """
        Used to return data from an ssh cmd callback method that can be used to handle output as it's rx'd instead of...
        waiting for the cmd to finish and returned buffer. See SshConnection.cmd() for more info.
        The call back must return type SshCbReturn.
        :param stop: If cb returns stop==True, recv loop will end, and channel will be closed, cmd will return.
        :param settimer: if cb settimer is > 0, timer timeout will be adjusted for this time
        :param statuscode: if cb statuscode is != -1 cmd status will return with this value
        :param nextargs: if cb nextargs is set, the next time cb is called these args will be passed instead
        :param buf: if cb buf is not None, the cmd['output'] buffer will be appended with this buf instead of std-out/err
        :param sendstring: ssh.cmd() will send this string to the channel if not None
        :param nextcb: optional callback can be return to ssh.cmd() to handle future output rx'd on the channel
        :param removecb: boolean used to remove any callback from future output for ssh.cmd()
        """
        self.stop = stop
        self.statuscode = statuscode
        self.settimer = settimer
        self.sendstring = sendstring
        self.nextargs = nextargs or []
        self.nextcb = nextcb
        self.removecb = removecb
        self.buf = buf


class SshConnection():
    cmd_timeout_err_code = -100
    cmd_not_executed_code = -99

    def __init__(self,
                 host,
                 username='root',
                 password=None,
                 keypair=None,
                 keypath=None,
                 proxy=None,
                 proxy_username='root',
                 proxy_password=None,
                 proxy_keyname=None,
                 proxy_keypath=None,
                 key_files=None,
                 find_keys=True,
                 enable_ipv6_dns=False,
                 timeout=60,
                 retry=1,
                 debugmethod=None,
                 verbose=False,
                 debug_connect=False):
        """
        :param host: -mandatory - string, hostname or ip address to establish ssh connection to
        :param username: - optional - string, username used to establish ssh session when keypath is not provided
        :param password: - optional - string, password used to establish ssh session when keypath is not provided
        :param keypair: - optional - boto keypair object, used to attept to derive local path to ssh key if present
        :param keypath:  - optional - string, path to ssh key
        :param proxy: - optional - host to proxy ssh connection through
        :param proxy_username:  - optional ssh username of proxy host for authentication
        :param proxy_password: - optional ssh password of proxy host for authentication
        :param proxy_keypath: - optional path to ssh key to use for proxy authentication
        :param key_files: - optional ',' comma delimited list of key file paths
        :param proxy_keyname: - optional keyname for proxy authentication, will attempt to derive keypath from this
        :param enable_ipv6_dns: - optional - boolean to allow ipv6 dns hostname resolution
        :param timeout: - optional - integer, tcp timeout in seconds
        :param retry: - optional - integer, # of attempts made to establish ssh session without auth failures
        :param debugmethod: - method, used to handle debug msgs
        :param verbose: - optional - boolean to flag debug output on or off mainly for cmd execution
        :param debug_connect: - optional - boolean to flag debug output on or off for connection related operations
        """

        self.host = host
        self.username = username
        self.password = password
        self.keypair = keypair
        self.keypath = keypath
        self.proxy = proxy
        self.proxy_username = proxy_username
        self.proxy_password = proxy_password
        self.proxy_keyname = proxy_keyname
        self.proxy_keypath = proxy_keypath
        self.enable_ipv6_dns = enable_ipv6_dns
        self.timeout = timeout
        self.retry = retry
        self.debugmethod = debugmethod
        self.verbose = verbose
        self.sftp = None
        self.key_files = key_files or []
        if not isinstance(self.key_files, types.ListType):
            self.key_files = str(self.key_files).split(',')
        self.find_keys = find_keys
        self.debug_connect = debug_connect

        #Used to store the last cmd attempted and it's exit code
        self.lastcmd = ""
        self.lastexitcode = SshConnection.cmd_not_executed_code

        if self.keypair is not None:
            self.keypath = os.getcwd() + "/" + self.keypair.name + ".pem"
        if self.keypath is not None:
            self.debug("SSH connection has hostname:" + str(self.host) + " user:" +
                       str(self.username) + " and keypath: " + str(self.keypath))
        else:
            self.debug("SSH connection has hostname:" + str(self.host) + " user:" +
                       str(self.username) + " password:" + str(self.mask_password(password)))
        if proxy:
            if self.proxy_keyname is not None:
                self.proxy_keypath = os.getcwd() + "/" + self.proxy_keyname + ".pem"
            if self.proxy_keypath is not None:
                self.debug("SSH proxy has hostname:" + str(self.proxy) + " user:" +
                           str(self.proxy_username) + " and keypath: " + str(self.proxy_keypath))
            else:
                self.debug("SSH proxy has hostname:" + str(self.proxy) + " user:" +
                           str(proxy_username) + " password:" + str(self.mask_password(proxy_password)))

        if self.find_keys or \
                self.keypath is not None or \
                ((self.username is not None) and (self.password is not None)):
            self.connection = self.get_ssh_connection(self.host,
                                                      username=self.username,
                                                      password=self.password,
                                                      keypath=self.keypath,
                                                      proxy_username=self.proxy_username,
                                                      proxy_password=self.proxy_password,
                                                      proxy_keypath=self.proxy_keypath,
                                                      enable_ipv6_dns=self.enable_ipv6_dns,
                                                      timeout=self.timeout,
                                                      retry=self.retry,
                                                      verbose=self.debug_connect)
        else:
            raise Exception("Need either a keypath or username+password to create ssh connection")


    def get_proxy_transport(self,
                            proxy_host=None,
                            dest_host=None,
                            port=22,
                            proxy_username='root',
                            proxy_password=None,
                            proxy_keypath=None,
                            key_files=None,
                            verbose=True):
        """


        :param key_files: pubkey file. If 'None' will check global self.key_files default:'~/.ssh/authorized_keys'
        :param verbose: print debug
        :param proxy_host: hostname of ssh proxy
        :param port: ssh proxy port
        :param dest_host: end host to connect to
        :param proxy_username: proxy username for ssh authentication
        :param proxy_password: proxy password for ssh authentication
        :param proxy_keypath: local path to key used for ssh authentication
        :return: paramiko transport
        """
        proxy_host = ((proxy_host or self.proxy),port)
        dest_host = ((dest_host or self.host),port)
        proxy_username = proxy_username or self.proxy_username
        proxy_password = proxy_password or self.proxy_password
        proxy_keypath = proxy_keypath or self.proxy_keypath
        key_files = key_files or self.key_files or []
        if key_files and not isinstance(key_files, types.ListType):
            key_files = key_files.split(',')

        #Make sure there is at least one likely way to authenticate...
        ssh = paramiko.SSHClient()
        if (proxy_username is not None) and (key_files or self.find_keys or proxy_keypath is not None or \
                         proxy_password is not None ):
            p_transport = paramiko.Transport(proxy_host)
            ssh._transport = p_transport
            p_transport.start_client()
            if proxy_keypath:
                priv_key = paramiko.RSAKey.from_private_key_file(proxy_keypath)
                p_transport.auth_publickey(proxy_username,priv_key)
            elif proxy_password:
                p_transport.auth_password(proxy_username, proxy_password)
            elif self.find_keys:
                self.debug("Proxy auth -Using local keys, no keypath/password provided",
                           verbose=verbose)
                ssh._auth(proxy_username, None,None,key_files, True, True)
                p_transport = ssh._transport
            #forward from 127.0.0.1:<free_random_port> to |dest_host|
            channel = p_transport.open_channel('direct-tcpip', dest_host, ('127.0.0.1', 0))
            return paramiko.Transport(channel)
        else:
            raise Exception("Need either a keypath or username+password to create ssh proxy connection")


    def debug(self, msg, verbose=None):
        """
        simple method for printing debug.
        :param msg: - mandatory - string to be printed
        :param verbose: boolean to override global verbose flag
        """
        if verbose is None:
            verbose = self.verbose
        if verbose is True:
            if self.debugmethod is None:
                print (str(msg))
            else:
                self.debugmethod(msg)

    def ssh_sys_timeout(self, chan, start, cmd):
        """
        callback to be scheduled during ssh cmds which have timed out.
        :param chan: - paramiko channel to be closed
        :param start - time.time() used to calc elapsed time when this fired for debug
        :param cmd - the command ran
        """
        chan.close()
        elapsed = time.time() - start
        raise CommandTimeoutException(
            "SSH Command timer fired after " + str(int(elapsed)) + " seconds. Cmd:'" + str(cmd) + "'")


    def sys(self, cmd, verbose=False, timeout=120, listformat=True, enable_debug=False, code=None):
        """
        Issue a command cmd and return output in list format

        :param cmd: - mandatory - string representing the command to be run  against the remote ssh session
        :param verbose: - optional - will default to global setting, can be set per cmd() as well here
        :param timeout: - optional - integer used to timeout the overall cmd() operation in case of remote blockingd
        :param listformat:  - optional - format output into single buffer or list of lines
        :param code: - optional - expected exitcode, will except if cmd's  exitcode does not match this value
        """
        out = self.cmd(cmd, verbose=verbose, timeout=timeout, listformat=listformat, enable_debug=enable_debug)
        output = out['output']
        if code is not None:
            if out['status'] != code:
                self.debug(output)
                raise CommandExitCodeException('Cmd:' + str(cmd) + ' failed with status code:'
                                               + str(out['status']) + ", output:" + str(output))
        return output


    def cmd(self, cmd, verbose=None, timeout=120, listformat=False, enable_debug=False, cb=None, cbargs=[], get_pty=True):
        """ 
        Runs a command 'cmd' within an ssh connection. 
        Upon success returns dict representing outcome of the command.

        Returns dict:
            ['cmd'] - The command which was executed
            ['output'] - The std out/err from the executed command
            ['status'] - The exitcode of the command. Note in the case a call back fires, this exitcode is unreliable.
            ['cbfired']  - Boolean to indicate whether or not the provided callback fired (ie returned False)
            ['elapsed'] - Time elapsed waiting for command loop to end. 
        Arguments:
        :param cmd: - mandatory - string representing the command to be run  against the remote ssh session
        :param verbose: - optional - will default to global setting, can be set per cmd() as well here
        :param timeout: - optional - integer used to timeout the overall cmd() operation in case of remote blocking
        :param listformat: - optional - boolean, if set returns output as list of lines, else a single buffer/string
        :param cb: - optional - callback, method that can be used to handle output as it's rx'd instead of...
                        waiting for the cmd to finish and return buffer. Called like: cb(ssh_cmd_out_buffer, *cbargs)
                        Must accept string buffer, and return an integer to be used as cmd status. 
                        Must return type 'sshconnection.SshCbReturn'
                        If cb returns stop, recv loop will end, and channel will be closed.
                        if cb settimer is > 0, timer timeout will be adjusted for this time
                        if cb statuscode is != -1 cmd status will return with this value
                        if cb nextargs is set, the next time cb is called these args will be passed instead of cbargs
        :param cbargs: - optional - list of arguments to be appended to output buffer and passed to cb
        :param enable_debug: - optional - boolean, if set will use self.debug() to print additional messages during cmd()

        """
        if verbose is None:
            verbose = self.verbose
        ret = {}
        cbfired = False
        cmd = str(cmd)
        self.lastcmd = cmd
        self.lastexitcode = SshConnection.cmd_not_executed_code
        start = time.time()
        status = None
        def cmddebug(msg):
            if enable_debug:
                self.debug(msg)
        if verbose:
            self.debug("[" + self.username + "@" + str(self.host) + "]# " + cmd)
        try:
            tran = self.connection.get_transport()
            if tran is None:
                self.debug("SSH transport was None, attempting to restablish ssh to: "+str(self.host))
                self.refresh_connection()
                tran = self.connection.get_transport()
            chan = tran.open_session()
            chan.settimeout(timeout)
            if get_pty:
                chan.get_pty()
            chan.exec_command(cmd)
            output = None
            fd = chan.fileno()
            chan.setblocking(0)
            cmdstart = start = time.time()
            newdebug = "\n"
            while not chan.closed:
                time.sleep(0.05)
                try:
                    rl, wl, xl = select.select([fd], [], [], timeout)
                except select.error:
                    break
                elapsed = int(time.time() - start)
                if elapsed >= timeout:
                    raise CommandTimeoutException(
                        "SSH Command timer fired after " + str(int(elapsed)) + " seconds. Cmd:'" + str(cmd) + "'")
                if len(rl) > 0:
                    cmddebug('ssh cmd: got input on recv channel')
                    while chan.recv_ready():
                        new = chan.recv(1024)
                        if verbose:
                            cmddebug('ssh cmd: got new data on channel:"' + str(new) + '"')
                        if new is not None:
                            #We have data to handle...
                            #Run call back if there is one, let call back handle data read in
                            if cb is not None:
                                if enable_debug:
                                    cbname = 'unknown'
                                    try:
                                        cbname = str(cb.im_func.func_code.co_name)
                                    except: pass
                                    self.debug('ssh cmd: sending new data to callback: ' + str(cbname))
                                #If cb returns false break, end rx loop, return cmd outcome/output dict. 
                                #cbreturn = SshCbReturn()
                                cbreturn = cb(new, *cbargs)
                                #Let the callback control whether or not to continue
                                if cbreturn.stop:
                                    cmddebug('ssh cmd: callback sent stop')
                                    if cbreturn.buf:
                                        if output is None:
                                            output = cbreturn.buf
                                        else:
                                            output += cbreturn.buf
                                    cbfired = True
                                    #Let the callback dictate the return code, otherwise -1 for connection err may occur
                                    if cbreturn.statuscode != -1:
                                        status = cbreturn.statuscode
                                    else:
                                        status = self.lastexitcode = chan.recv_exit_status()
                                    chan.close()
                                    break
                                else:

                                    #Let the callback update its calling args if needed
                                    if cbreturn.nextargs is not None:
                                        cbargs = cbreturn.nextargs
                                    #Let the callback update/reset the timeout if needed
                                    if cbreturn.settimer > 0:
                                        start = time.time()
                                        timeout = cbreturn.settimer
                                    #Let the callback update the output buffer to be returned
                                    if cbreturn.buf:
                                        cmddebug('ssh cmd: cb returned buf:"' + str(cbreturn.buf) + '"')
                                        if output is None:
                                            output = cbreturn.buf
                                        else:
                                            output += cbreturn.buf
                                    #Change the callback to handle future output from this cmd
                                    if cbreturn.nextcb:
                                        cmddebug('ssh cmd: updating to new callback provided in cb return nextcb')
                                        cb = cbreturn.nextcb
                                    #Remove all callbacks
                                    if cbreturn.removecb:
                                        cmddebug('ssh cmd: removing all callbacks per cb return removecb value')
                                        cb = None
                                    #Send a string to the channel provided in callback (similar to expect)
                                    if cbreturn.sendstring is not None:
                                        if verbose:
                                            cmddebug('Sending string:' + str(cbreturn.sendstring))
                                        chan.send(s=str(cbreturn.sendstring))
                                        cmddebug('channel status after sending string. Is closed = ' + str(chan.closed))
                            else:
                                #if no call back then append output to return dict and handle debug
                                if output == None:
                                    output = new
                                else:
                                    output += new
                                if verbose:
                                    #Dont print line by line output if cb is used, let cb handle that 
                                    newdebug += new
                        else:
                            status = self.lastexitcode = chan.recv_exit_status()
                            chan.close()
                            break
                    if newdebug and verbose:
                        self.debug(str(newdebug))
                        newdebug = ''
                elif enable_debug:
                    self.debug('ssh cmd: len of rl was < 0')
            cmddebug('ssh cmd: channel closed')
            if output is None:
                output = ""
            if listformat:
                #return output as list of lines
                output = output.splitlines()
                if output is None:
                    output = []

            #add command outcome in return dict.
            if status is None:
                status = self.lastexitcode = chan.recv_exit_status()
            ret['cmd'] = cmd
            ret['output'] = output
            ret['status'] = status
            ret['cbfired'] = cbfired
            ret['elapsed'] = int(time.time() - cmdstart)
            if verbose:
                self.debug("done with exec")
        except CommandTimeoutException, cte:
            self.lastexitcode = SshConnection.cmd_timeout_err_code
            elapsed = str(int(time.time() - start))
            self.debug("Command (" + cmd + ") timeout exception after " + str(elapsed) + " seconds\nException")
            raise cte
        return ret

    def refresh_connection(self):
        """
        Attempts to establish a new ssh connection to replace the old 'connection' of this
        ssh obj.
        """
        if self.connection:
            self.connection.close()
        self.connection = self.get_ssh_connection(self.host,
                                                  username=self.username,
                                                  password=self.password,
                                                  keypath=self.keypath,
                                                  proxy_username=self.proxy_username,
                                                  proxy_password=self.proxy_password,
                                                  proxy_keypath=self.proxy_keypath,
                                                  enable_ipv6_dns=self.enable_ipv6_dns,
                                                  timeout=self.timeout,
                                                  retry=self.retry,
                                                  verbose=self.debug_connect)

    def get_ssh_connection(self,
                           hostname,
                           username="root",
                           password=None,
                           keypath=None,
                           proxy=None,
                           proxy_username=None,
                           proxy_password=None,
                           proxy_keypath=None,
                           key_files=None,
                           enable_ipv6_dns=None,
                           port=22,
                           timeout=60,
                           retry=1,
                           verbose=False):
        """
        Create a paramiko ssh session to hostname. Will attempt to authenticate first with a keypath if provided,
        if the sshkey file path is not provided.  username and password will be used to authenticate. This leaves out
        the case where a password is passed as the password needed to unlock the key file. This 3rd case may need to be
        added but may mask failures in tests for key insertion when using tests who's images have baked in passwords for
        login access(tbd).
        Upon success returns a paramiko sshclient with an established connection.

        :param hostname: - mandatory - hostname or ip to establish ssh connection with
        :param username: - optional - username used to authenticate ssh session
        :param password: - optional - password used to authenticate ssh session
        :param keypath: - optional - full path to sshkey file used to authenticate ssh session
        :param proxy: - optional - host to proxy ssh connection through
        :param proxy_username:  - optional ssh username of proxy host for authentication
        :param proxy_password: - optional ssh password of proxy host for authentication
        :param proxy_keypath: - optional path to ssh key to use for proxy authentication
        :param timeout: - optional - tcp timeout
        :param enable_ipv6_dns: - optional - boolean to avoid ipv6 dns 'AAAA' lookups
        :param retry: - optional - Number of attempts to establish ssh connection for errors outside of authentication
        :param port: - optional - port to connect to, default 22
        :param verbose: - optional - enable verbose debug output
        """
        connected = False
        iplist = []
        ip = None
        key_files = key_files or self.key_files or []
        if key_files and not isinstance(key_files, types.ListType):
            key_files = key_files.split(',')
        proxy_ip = None
        if not key_files and password is None and keypath is None and not self.find_keys:
            raise Exception("ssh_connect: Need to set password, keypath, keyfiles, or find_keys")
        if enable_ipv6_dns is None:
            enable_ipv6_dns = self.enable_ipv6_dns
        proxy = proxy or self.proxy

        self.debug("ssh_connect args:\nhostname:" + hostname
                    + "\nusername:" + username
                    + "\npassword:" + str(password)
                    + "\nkeypath:" + str(keypath)
                    + "\nproxy_username:" + str(proxy_username)
                    + "\nproxy_password" + str(proxy_password)
                    + "\nproxy_keypath" + str(proxy_keypath)
                    + "\ntimeout:" + str(timeout)
                    + "\nretry:" + str(retry),verbose=verbose)
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        hostname = str(hostname.strip())
        if not enable_ipv6_dns:
            self.debug('IPV6 DNS lookup disabled, do IPV4 resolution and pass IP to connect()',verbose=verbose)
            # Paramiko uses family type 'AF_UNSPEC' which does both ipv4/ipv6 lookups and can cause some DNS servers
            # to fail in their response(s). Hack to avoid ipv6 lookups...
            # Try ipv4 dns resolution of 'hostname', and pass the ip instead of a hostname to
            # Paramiko's connect to avoid the potential ipv6 'AAAA' lookup...
            iplist = self.get_ipv4_lookup(hostname,verbose=verbose)
        if not iplist:
            iplist = [hostname]
        attempt = 0
        #adjust retry count for debug 'readability' ie 'attempt 1' vs 'attempt 0'
        retry += 1
        while (attempt < retry) and not connected:
            attempt += 1
            proxy_transport = None
            for ip in iplist:
                if self.proxy:
                    if not enable_ipv6_dns:
                        proxy_ip = self.get_ipv4_lookup(self.proxy, verbose=verbose)[0]
                        proxy_transport = self.get_proxy_transport(proxy_host=proxy,
                                                                   dest_host=ip,
                                                                   port=port,
                                                                   proxy_username=proxy_username,
                                                                   proxy_password=proxy_password,
                                                                   proxy_keypath=proxy_keypath)
                if proxy_transport:
                    ssh._transport = proxy_transport
                else:
                    ssh._transport = paramiko.Transport(ip)
                ssh._transport.start_client()
                ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
                try:
                    self.debug("SSH connection attempt(" + str(attempt) +" of " + str(retry) + "), host:'"
                               + username + "@" + hostname + "', using ipv4:" + str(ip) + ", thru proxy:'"
                               + str(proxy_ip) + "'")
                    if keypath is None and password:
                        self.debug("Using username:"+username+" and password:"+str(self.mask_password(password)),
                                   verbose=verbose)
                        ssh._transport.auth_password(username, password)
                        #ssh.connect(ip, username=username, password=password, timeout=timeout)
                        connected = True
                        break
                    elif keypath:
                        self.debug("Using Keypath:" + keypath, verbose=verbose)
                        priv_key = paramiko.RSAKey.from_private_key_file(keypath)
                        ssh._transport.auth_publickey(username,priv_key)
                        #ssh.connect(ip, port=port, username=username, key_filename=keypath, timeout=timeout)
                        connected = True
                        break
                    elif key_files or self.find_keys:
                        self.debug("Using local keys, no keypath/password provided.", verbose=verbose)
                        ssh._auth(username, password,None,key_files, True, True)
                        #ssh.connect(ip, port=port, username=username, key_filename=keypath, timeout=timeout)
                        connected = True

                except paramiko.ssh_exception.SSHException, se:
                    self.debug("Failed to connect to " + hostname + ", retry in 10 seconds. Err:" + str(se))
                    time.sleep(10)
                    pass
            if connected:
                via_string = ''
                if proxy_transport:
                    proxy_host,port = ssh._transport.getpeername()
                    via_string = ' via proxy host:'+str(proxy_host)+':'+str(port)
                self.debug('SSH - Connected to ' + str(ip)+str(via_string))
                break
        if not connected:
            raise Exception(
                'Failed to connect to "' + str(hostname) + '", attempts:' + str(attempt) + ". IPs tried:" + ",".join(
                    iplist))
            #self.debug("Returning ssh connection to: "+ hostname)
        return ssh

    def get_ipv4_lookup(self, hostname, port=22,verbose=False):
        """
        Do an ipv4 lookup of 'hostname' and return list of any resolved ip addresses

        :param hostname: hostname to resolve
        :param port: port to include in lookup, default is ssh port 22
        :param verbose: boolean to print addditional debug
        :return: list of ip addresses (strings in a.b.c.d format)
        """
        get_ipv4_ip = False
        iplist = []
        try:
            if socket.inet_aton(hostname):
                ipcheck = re.compile("^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$")
                if not ipcheck.match(hostname):
                    get_ipv4_ip = True
            self.debug(str(hostname) + ", is already an ip, dont do host lookup...", verbose=verbose)
            # This is already an ip don't look it up (regex might be better here?)
        except socket.error:
            get_ipv4_ip = True
        if get_ipv4_ip:
            try:
                #ipv4 lookup host for ssh connection...
                addrs = socket.getaddrinfo(hostname, port, socket.AF_INET, socket.IPPROTO_IP, socket.IPPROTO_TCP)
                for addr in addrs:
                    iplist.append(str(addr[4][0]))
                self.debug('Resolved hostname:' + str(hostname) + ' to IP(s):' + ",".join(iplist),verbose=verbose)
            except Exception, de:
                self.debug('Error looking up DNS ip for hostname:' + str(hostname) + ", err:" + str(de))
        else:
            #hostname is an ipv4 address...
            iplist = [hostname]
        return iplist

    def mask_password(self,pass_string):
        """
        Replace all but first and last chars with '*' of provided password string.

        :param pass_string: string representing a password to hide/format
        :return: Formatted hidden password
        """
        password = copy.copy(pass_string)
        show = ""
        if not password:
            return password
        if len(password) > 3:
            length = len(password)-2
        else:
            length = len(password)
        for x in xrange(length):
            show += '*'
        if len(password) > 3:
            show = password[0]+show
            show += password[len(password)-1]
        return show

    def expect_password_cb(self,
                           buf,
                           password,
                           prompt="^Password",
                           nextcb=None,
                           cbargs=[],
                           retry=0,
                           password_attempts=0,
                           verbose=False):
        '''
        Sample callback to handle password prompts to be provided to ssh.cmd()
        :param buf: output from cmd()
        :param password: string password to be supplied to a detected 'password' prompt
        :param nextcb: function/method callback to be returned, this cmd() will use to handle it's future output.
        :param prompt: regex string used to match prompt. case insensitive match used
        :
        '''
        ret = SshCbReturn(stop=False)
        #newbuf = None
        def debug(msg, ssh=self):
            if verbose:
                ssh.debug(msg)

        def add_to_buffer(lines_to_add, newbuf):
            for line in lines_to_add:
                debug('Adding line to buf:"' + str(line) +'"')
                if newbuf is None:
                    newbuf = line+ '\n'
                else:
                    newbuf += line + '\n'
            return newbuf
        bufadd = lambda line: add_to_buffer(line, ret.buf)
        debug('STARTING expect_password_cb: password:' + str(password)+", prompt:"+str(prompt))
        debug('Starting buf:"' + str(buf) + '"')
        #Create a callback return obj for the cmd() loop to consume...

        lines = buf.splitlines()

        #See if we've already gone through password dialog, if so there may be a left over newline. Remove it.
        if password_attempts and lines[0] == "":
            debug('Removing first blank line(s) after sending password')
            lines.pop(0)
            if not lines:
                #self.debug('Skipping blanklines...')
                ret.buf = None
                ret.nextargs=[password, prompt, nextcb, cbargs, retry, password_attempts, verbose]
                return ret

        #See if we have a prompt for password, assume we only have one match and were blocking waiting on password input
        prompt_indices = [i for i, s in enumerate(lines) if re.match(prompt, s, re.IGNORECASE)]
        if prompt_indices:
            debug('Got password prompt, sending password...')
            #Check to see if we've already tried a password, and if we should retry or fail
            if password_attempts > retry:
                raise CommandExpectPasswordException("Password dialog attempts:" + str(password_attempts) +
                                                     " exceeded retry limit:" + str(retry))
            prompt_index = prompt_indices[0]
            #Add any lines other than password prompt back to return buffer
            #Remove line with password prompt first
            lines.pop(prompt_index)
            ret.buf = bufadd(lines)
            #Add password to CbReturn sendstring value to be sent to channel in cmd() loop...
            ret.sendstring = str(password).rstrip() + "\n"
            #Increment our password attempts value, and args to return obj for next time we get called. Expecting to
            # get called at least 1 more time to handle the blank line after password dialog, may also get called for a
            # login, and prompted for a password again...
            password_attempts += 1
            ret.removecb = False
            ret.nextcb = None
            ret.nextargs=[password, prompt, nextcb, cbargs, retry, password_attempts, verbose]
            debug('Ending buf:"' + str(ret.buf) + '"')
            return ret
        else:
            debug('\nPassword prompt not found, continuing. password_attempts:'+ str(password_attempts) +
                       ', prompt:' + str(prompt) + ', len lines: ' + str(len(lines)))
            #Assume the password dialog is complete, pass buffer to next handler,
            # attempt to make password portion look transparent
            ret.buf = bufadd(lines)
            if nextcb is not None:
                debug('Got nextcb, calling it on our buffer now...')
                ret = nextcb(ret.buf, *cbargs)
                if ret.nextcb and not ret.removecb:
                    nextcb=ret.nextcb
                else:
                    nextcb=None
            #tweak the return values, store the real ones in our nextargs and handle things in this method instead
            ret.nextcb = None
        ret.removecb = False
        ret.nextargs = [password, prompt, nextcb, cbargs, retry, password_attempts, verbose]
        debug('Ending buf:"' + str(ret.buf) + '"')
        return ret


    def start_interactive(self, timeout=180):
        '''
        Example method to invoke an interactive shell
        :pararm timeout: inactive session timeout, a value of 0 will wait for input/output forever
        '''
        tran = self.connection.get_transport()
        if tran is None:
            self.debug("SSH transport was None, attempting to re-establish ssh to: "+str(self.host))
            self.refresh_connection()
            tran = self.connection.get_transport()
        chan = tran.open_session()
        chan.get_pty()
        chan.setblocking(0)
        print('Opened channel, starting interactive mode...')
        oldtty = termios.tcgetattr(sys.stdin)
        try:
            tty.setraw(sys.stdin.fileno())
            tty.setcbreak(sys.stdin.fileno())
            chan.settimeout(0)
            chan.invoke_shell()
            fd = chan.fileno()
            while True:
                time.sleep(0.05)
                try:
                    read_ready, wlist, xlist = select.select([fd, sys.stdin], [], [],timeout)
                except select.error, se:
                    print 'select error:' + str(se)
                    break
                if fd in read_ready:
                    try:
                        recv = chan.recv(1024)
                        if recv is None or len(recv) == 0:
                            self.debug('Session closing (chan)...   ')
                            break
                        sys.stdout.write(recv)
                        sys.stdout.flush()
                    except socket.timeout:
                        pass
                elif sys.stdin in read_ready:
                    user_input = sys.stdin.read(1)
                    if user_input is None or len(user_input) == 0:
                        self.debug('Session closing (stdin)...')
                        break
                    chan.send(user_input)
                else:
                    self.debug('Got nothing, closing...')
                    break
        finally:
            try:
                termios.tcsetattr(sys.stdin, termios.TCSADRAIN, oldtty)
            except:pass
            if chan:
                chan.close()



    def open_sftp(self, transport=None):
        transport = transport or self.connection._transport
        self.sftp = paramiko.SFTPClient.from_transport(transport)

    def close_sftp(self):
        self.sftp.close()



    def sftp_put(self,localfilepath,remotefilepath):
        """
        sftp transfer file from localfilepath to remote system at remotefilepath
        :param localfilepath: path to file on local system
        :param remotefilepath: destination path for put on remote system
        """
        if not self.connection._transport:
            self.refresh_connection()
        transport = self.connection._transport
        self.open_sftp()
        self.sftp.put(remotepath=remotefilepath, localpath=localfilepath)
        self.close_sftp()

    def sftp_get(self, localfilepath, remotefilepath):
        """
        sftp transfer file from remotefilepath to remote system at localfilepath
        :param localfilepath: path where remote file 'get' will place file on local system
        :param remotefilepath: destination path for file to 'get' on remote system
        """
        if not self.connection._transport:
            self.refresh_connection()
        transport = self.connection._transport
        self.open_sftp()
        self.sftp.get(remotepath=remotefilepath, localpath=localfilepath)
        self.close_sftp()


    def close(self):
        self.connection.close()


class CommandExitCodeException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)

class CommandTimeoutException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)

class CommandExpectPasswordException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)




#!/usr/bin/python
from eutester.eutestcase import EutesterTestCase, EutesterTestUnit, EutesterTestResult
from testcases.cloud_user.images.windows.windowstests import WindowsTests
from eutester.euconfig import EuConfig
from eutester.machine import Machine


machine=None
testcase = EutesterTestCase()

testcase.setup_parser(testname='load_windows_image.py', 
                      description='Loads a windows image from either a remote url or local file path', 
                      emi=False,
                      testlist=False)

testcase.parser.add_argument('--url',help='URL containing remote windows image to create EMI from', default=None)
testcase.parser.add_argument('--file',dest='image_file_path', help='File path to create windows EMI from', default=None)
testcase.parser.add_argument('--workip',help='The IP of the machine that the operation will be performed on', default=None)
testcase.parser.add_argument('--destpath',help='The path on the workip, that this operation will be performed on', default='/disk1/storage')
testcase.parser.add_argument('--urlpass', dest='wget_password',help='Password needed to retrieve remote url', default=None)
testcase.parser.add_argument('--urluser',dest='wget_user', help='Username needed to retrieve remote url', default=None)
testcase.parser.add_argument('--gigtime',dest='time_per_gig', help='Time allowed per gig size of image to be used', default=300)
testcase.parser.add_argument('--interbundletime',dest='inter_bundle_timeout', help='Inter-bundle timeout', default=120)
testcase.parser.add_argument('--virtualization_type', help='bucketname', default=None)
testcase.parser.add_argument('--bucket',dest='bucketname', help='bucketname', default=None)

testcase.get_args()

if (not testcase.args.url and not testcase.args.image_file_path) or (testcase.args.url and testcase.args.image_file_path):
    raise Exception('Must specify either a URL or FILE path to create Windows EMI from')
if testcase.args.workip:
    machine = Machine(hostname=testcase.args.workip,password=testcase.args.password)

WinTests = testcase.do_with_args(WindowsTests,work_component=machine)

if testcase.args.image_file_path:
    test = testcase.create_testunit_from_method(WinTests.create_windows_emi_from_file)
else:
    test = testcase.create_testunit_from_method(WinTests.create_windows_emi_from_url)

testcase.run_test_case_list([test], eof=True, clean_on_exit=False, printresults=True)





# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com

from eutester import Eutester
import os
import hashlib
from boto.s3.connection import OrdinaryCallingFormat
from boto.s3.key import Key
from boto.s3.acl import ACL, Grant
from boto.exception import S3ResponseError
from boto.s3.deletemarker import DeleteMarker
import boto.s3

class S3opsException(Exception):
    """Exception raised for errors that occur when running S3 operations.

    Attributes:
        msg  -- explanation of the error
    """
    def __init__(self, msg):
        self.msg = msg
    
    def __str__(self):
        print self.msg

class S3ops(Eutester):
    s3_groups = {
             "all_users":"http://acs.amazonaws.com/groups/global/AllUsers",
             "authenticated_users":"http://acs.amazonaws.com/groups/global/AuthenticatedUsers",
             "log_delivery":"http://acs.amazonaws.com/groups/s3/LogDelivery"
             }

    def __init__(self, endpoint=None, credpath=None, aws_access_key_id=None, aws_secret_access_key = None, is_secure=False, path="/", port=80, boto_debug=0):
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.user_id = None
        self.account_id = None
        super(S3ops, self).__init__(credpath=credpath)
        self.setup_s3_connection(endpoint=endpoint, aws_access_key_id=self.aws_access_key_id ,aws_secret_access_key=self.aws_secret_access_key, is_secure=is_secure, path=path, port=port, boto_debug=boto_debug)
        self.test_resources = {}
        self.setup_s3_resource_trackers()

    def setup_s3_connection(self, endpoint=None, aws_access_key_id=None, aws_secret_access_key=None, is_secure=False, path="/", port=80, boto_debug=0):
        try:
            if not endpoint:
                endpoint = self.get_s3_ip()
            s3_connection_args = { 'aws_access_key_id' :aws_access_key_id,
                                   'aws_secret_access_key': aws_secret_access_key,
                                   'is_secure': is_secure,
                                   'host' : endpoint,
                                   'path'  : path,
                                   'port' : port,
                                   'debug':boto_debug,
                                   'calling_format':OrdinaryCallingFormat(),
                                   }
            self.debug("Attempting to create S3 connection to " + endpoint + ':' + str(port) + path)
            self.s3 = boto.connect_s3(**s3_connection_args)
        except Exception, e:
            raise Exception("Was unable to create S3 connection because of exception: " + str(e))

    def setup_s3_resource_trackers(self):
        """
        Setup keys in the test_resources hash in order to track artifacts created
        """
        self.test_resources["keys"] = []
        self.test_resources["buckets"] = []

    def get_s3_ip(self):
        """Parse the eucarc for the S3_URL"""
        walrus_url = self.parse_eucarc("S3_URL")
        return walrus_url.split("/")[2].split(":")[0]

    def create_bucket(self,bucket_name):
        """
        Create a bucket.  If the bucket already exists and you have
        access to it, no error will be returned by AWS.
        Note that bucket names are global to S3
        so you need to choose a unique name.
        """
        # First let's see if we already have a bucket of this name.
        # The lookup method will return a Bucket object if the
        # bucket exists and we have access to it or None.
        bucket = self.get_bucket_by_name(bucket_name)
        if bucket:
            self.debug( 'Bucket (%s) already exists' % bucket_name )
        else:
                # Let's try to create the bucket.  This will fail if
                # the bucket has already been created by someone else.
            try:
                bucket = self.s3.create_bucket(bucket_name)
            except self.s3.provider.storage_create_error, e:
                raise S3opsException( 'Bucket (%s) is owned by another user' % bucket_name )
            if not self.get_bucket_by_name(bucket.name):
                raise S3opsException("Bucket could not be found after creation")
        self.test_resources["buckets"].append(bucket)
        self.debug("Created bucket: " + bucket_name)
        return bucket
    
    def delete_bucket(self, bucket):
        """
        Delete a bucket.
        bucket_name  The name of the Walrus Bucket
        """
        # First let's see if we already have a bucket of this name.
        # The lookup method will return a Bucket object if the
        # bucket exists and we have access to it or None.
        bucket_name = bucket.name
        try:
            bucket.delete()
        except self.s3.provider.storage_create_error, e:
                raise S3opsException( 'Bucket (%s) is owned by another user' % bucket_name )
        ### Check if the bucket still exists
        if self.get_bucket_by_name(bucket_name):
            raise S3opsException('Bucket (%s) still exists after delete operation'  % bucket_name )
    
    def get_bucket_by_name(self, bucket_name):
        """
        Lookup a bucket by name, if it does not exist raise an exception
        """
        bucket = self.s3.lookup(bucket_name)
        if bucket:
            return bucket
        else:
            return None
    
    def upload_object(self, bucket_name, key_name, path_to_file=None, contents=None):
        """
        Write the contents of a local file to walrus
        bucket_name   The name of the walrus Bucket.
        key_name      The name of the object containing the data in walrus.
        path_to_file  Fully qualified path to local file.
        """
        bucket = self.get_bucket_by_name(bucket_name)
        if bucket == None:
            raise S3opsException("Could not find bucket " + bucket_name + " to upload file")
        # Get a new, blank Key object from the bucket.  This Key object only
        # exists locally until we actually store data in it.
        key = bucket.new_key(key_name)
        if key == None:
            self.fail( "Unable to create key " + key_name  )
        if path_to_file is None:
            if contents is None:
                contents = os.urandom(1024)
            key.set_contents_from_string(contents)
        else:
            key.set_contents_from_filename(path_to_file)
        self.debug("Uploaded key: " + str(key_name) + " to bucket:" + str(bucket_name))    
        self.test_resources["keys"].append(key)
        return key
    
    def get_objects_by_prefix(self, bucket_name, prefix):
        """
        Get keys in the specified bucket that match the prefix if no prefix is passed all objects are returned
        as a result set.
        If only 1 key matches it will be returned as a Key object. 
        """
        bucket = self.get_bucket_by_name(bucket_name)
        keys = bucket.get_all_keys(prefix=prefix)
        if len(keys) < 1:
            self.fail("Unable to find any keys with prefix " + prefix + " in " + str(bucket) )
        if len(keys) == 2:
            return keys[0]
        return keys
        
    def delete_object(self, object):
        bucket = object.bucket
        name = object.name
        object.delete()
        try:
            self.s3.get_bucket(bucket).get_key(name)
            raise S3opsException("Walrus object " + name + " in bucket "  +  bucket.name  + " still exists after delete")
        except Exception, e:
            return
        
    def clear_bucket(self, bucket_name=None):
        """Deletes the contents of the bucket specified and the bucket itself
            THIS WILL DELETE EVERYTHING!
           bucket       bucket name to clear
        """
        try :
            bucket = self.s3.get_bucket(bucket_name=bucket_name)      
        except S3ResponseError as e:
            self.error('No bucket' + bucket_name + ' found: ' + e.message)
            raise Exception('Not found')
        
        try:
            self.debug( "Getting bucket listing for " + bucket.name )     
            self.debug(  "Iterating throught the bucket" )
            key_list = bucket.list()        
            self.debug(  "Starting loop" )
            for k in key_list:
                if isinstance(k, boto.s3.prefix.Prefix):
                    self.debug(  "Skipping prefix" )
                    continue
                self.debug(  "Deleting key: " + k.name )
                bucket.delete_key(k)
            bucket.delete()
        except S3ResponseError as e:
            self.debug(  "Exception caught doing bucket cleanup." )
            #Todo: need to make this work with Walrus's non-S3-compliant error codes
            if e.status == 409:
                #Do version cleanup
                self.debug(  "Cleaning up versioning artifacts" )
                try:
                    keys = bucket.get_all_versions()
                    for k in keys:
                        if isinstance(k, Key):
                            self.debug(  "Got version: " + k.name + "--" + k.version_id + "-- Delete marker? " + str(k.delete_marker) )
                            self.debug(  "Deleting key: " + k.name )
                            bucket.delete_key(key_name=k.name,version_id=k.version_id)
                        elif isinstance(k, DeleteMarker):
                            self.debug(  "Got marker: " + k.name + "--" + k.version_id + "--" + str(k.is_latest) )
                            self.debug(  "Deleting delete marker" )
                            bucket.delete_key(key_name=k.name,version_id=k.version_id)
                    self.debug(  "Deleting bucket " + bucket.name )
                    bucket.delete()
                except Exception as e:
                    self.debug(  "Exception deleting versioning artifacts: " + e.message )
            else:
                self.debug('Got ' + e.message + ' and status ' + str(e.status))
                    
    def clear_keys_with_prefix(self, bucket, prefix):
        try :
            listing = self.walrus.get_all_buckets()        
            for bucket in listing:
                if bucket.name.startswith(prefix):
                    self.debug( "Getting bucket listing for " + bucket.name)
                    key_list = bucket.list()
                    for k in key_list:
                        if isinstance(k, boto.s3.prefix.Prefix):
                            self.debug( "Skipping prefix" )
                            continue
                        self.debug( "Deleting key: " + k.name )
                        bucket.delete_key(k)
                    bucket.delete()
                else:
                    self.debug( "skipping bucket: " + bucket.name )
        except S3ResponseError as e:
            raise S3opsException( "Exception caught doing bucket cleanup." )
                    
    
    def get_canned_acl(self, canned_acl=None, bucket_owner_id=None, bucket_owner_display_name=None):
        '''
        Returns an acl object that can be applied to a bucket or key. It is intended to be used to verify
        results that the service returns. To set a canned-acl you can simply set it on the bucket directly without
        this method.
        
        bucket_owner_id         Account id of the owner of the bucket. Required
        canned_acl       Canned acl to implement. Required. 
                         Options: ['private','public-read', 'public-read-write', 'authenticated-read',  'log-delivery-write', 'bucket-owner-full-control', 'bucket-owner-full-control']
        bucket_owner_display_name  Required. The account display name for the bucket owner, so that the correct permission can be generated fully
        '''
        if bucket_owner_id == None or canned_acl == None or bucket_owner_display_name == None :
            raise S3opsException( "No user_id or canned_acl passed to get_canned_acl()" )
        
        built_acl = ACL()
        built_acl.add_user_grant(permission='FULL_CONTROL',user_id=bucket_owner_id, display_name=bucket_owner_display_name)
        
        if canned_acl == "public-read":
            built_acl.add_grant(Grant(permission="READ",type='Group',uri=self.s3_groups["all_users"]))        
        elif canned_acl == "public-read-write":
            built_acl.add_grant(Grant(permission="READ",type='Group',uri=self.s3_groups["all_users"]))
            built_acl.add_grant(Grant(permission="WRITE",type='Group',uri=self.s3_groups["all_users"]))                
        elif canned_acl == "authenticated-read":
            built_acl.add_grant(Grant(permission="READ",type='Group',uri=self.s3_groups["authenticated_users"]))        
        elif canned_acl == "log-delivery-write":
            built_acl.add_grant(Grant(permission="WRITE",type='Group',uri=self.s3_groups["log_delivery"]))        
        elif canned_acl == "bucket-owner-read":
            if bucket_owner_id is None:
                raise Exception("No bucket_owner_id passed when trying to create bucket-owner-read canned acl ")
            built_acl.add_grant(Grant(permission="READ",id=bucket_owner_id))
        elif canned_acl == "bucket-owner-full-control":
            if bucket_owner_id is None:
                raise Exception("No bucket_owner_id passed when trying to create bucket-owner-full-control canned acl ")
            built_acl.add_grant(Grant(permission="FULL_CONTROL",id=bucket_owner_id))
        return built_acl
    
    def check_acl_equivalence(self, acl1=None, acl2=None):
        '''
        Checks if acl1 = acl2 based on comparison of the set of grants irrespective of order.
        One limitation is that each grant's xml string deserialization must be the same to be
        considered equivalent. This has implications for the grant displayname in particular.
        For example, an ACL with an unknown account specified will not generally have a
        display-name associated with the account id, so the comparison may fail in that case even
        though the ids and permissions are identical.
        
        Returns None if there is an input problem such as one or more inputs are None
        
        acl1    An ACL object from boto.s3.acl
        acl2    An ACL object from boto.s3.acl
        '''
        if acl1 == None or acl2 == None:
            return None
        
        acl1grants = set()
        acl2grants = set()
        
        #calculate the symmetric-difference of the two sets of grants
        for val in acl1.grants:
            acl1grants.add(val.to_xml())
        
        for val in acl2.grants:
            acl2grants.add(val.to_xml())        
            
        return not len(acl1grants.symmetric_difference(acl2grants)) > 0

    def check_md5(self, eTag=None, data=None):
        hasher = hashlib.md5()
        hasher.update(data)
        data_hash = hasher.hexdigest()
        if data_hash != eTag:
            raise Exception( "Hash/eTag mismatch: \nhash = " + data_hash + "\neTag= " + eTag )
            
                


#!/usr/bin/env python

#
####################################
#                                  #
#   Walrus/S3 Bucket Test Cases    #
#                                  #
####################################

#Author: Zach Hill <zach@eucalyptus.com>

from eucaops import Eucaops
import argparse
import time
import re

from eutester.eutestcase import EutesterTestCase
from eucaops import S3ops
from boto.exception import S3ResponseError
from boto.exception import S3CreateError
import boto
import boto.s3
from boto.s3.bucket import Bucket
from boto.s3.key import Key
from boto.s3.acl import ACL, Policy, Grant
from boto.s3.connection import Location


class BucketTestSuite(EutesterTestCase):
    
    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("--s3endpoint", default=None)
        self.get_args()
        # Setup basic eutester object
        if self.args.s3endpoint:
            self.tester = S3ops( credpath=self.args.credpath, endpoint=self.args.endpoint)
        else:
            self.tester = Eucaops( credpath=self.args.credpath, config_file=self.args.config, password=self.args.password)

        self.bucket_prefix = "eutester-bucket-test-suite-" + str(int(time.time())) + "-"
        self.buckets_used = set()
        
    def test_bucket_get_put_delete(self):
        '''
        Method: Tests creating and deleting buckets as well as getting the bucket listing
        '''
        test_bucket=self.bucket_prefix + "simple_test_bucket"
        self.buckets_used.add(test_bucket)
        self.tester.debug("Starting get/put/delete bucket test using bucket name: " + test_bucket)
 
        try :
            bucket = self.tester.s3.create_bucket(test_bucket)                
            if bucket == None:
                self.tester.s3.delete_bucket(test_bucket)
                self.fail(test_bucket + " was not created correctly")
        except (S3ResponseError, S3CreateError) as e:
            self.fail(test_bucket + " create caused exception: " + e)
        
        try :    
            bucket = self.tester.s3.get_bucket(test_bucket)
            if bucket == None:
                self.tester.s3.delete_bucket(test_bucket)
                self.fail(test_bucket +" was not fetched by get_bucket call")
        except S3ResponseError as e:
            self.tester.s3.delete_bucket(test_bucket)
            self.fail("Exception getting bucket" + e)
            
        
        self.tester.s3.delete_bucket(test_bucket)        
        try :
            if self.tester.s3.get_bucket(test_bucket) != None:
                self.tester.s3.delete_bucket(test_bucket)            
                self.fail("Delete of " + test_bucket + " failed, still exists")
        except S3ResponseError as e:
            self.tester.debug( "Correctly got exception trying to get a deleted bucket! " )
            
        self.tester.debug( "Testing an invalid bucket names, calls should fail." )
        try:
            bad_bucket = self.bucket_prefix + "bucket123/"
            self.tester.create_bucket(bad_bucket)
            should_fail = True            
            try:
                self.tester.delete_bucket(bad_bucket)
            except:
                self.tester.debug( "Exception deleting bad bucket, shouldn't be here anyway. Test WILL fail" )
                
            if should_fail:
                self.fail("Should have caught exception for bad bucket name: " + bad_bucket)
        except:
            self.tester.debug( "Correctly caught the exception" )
        
        try:
            bad_bucket = self.bucket_prefix + "bucket.123"
            self.tester.create_bucket(bad_bucket)
            should_fail = True            
            try:
                self.tester.delete_bucket(bad_bucket)
            except:
                self.tester.debug( "Exception deleting bad bucket, shouldn't be here anyway. Test WILL fail" )
                
            if should_fail:
                self.fail("Should have caught exception for bad bucket name: " + bad_bucket)
        except:
            self.tester.debug( "Correctly caught the exception" )
        
        try:
            bad_bucket = self.bucket_prefix + "bucket&123"
            self.tester.create_bucket(bad_bucket)
            should_fail = True            
            try:
                self.tester.delete_bucket(bad_bucket)
            except:
                self.tester.debug( "Exception deleting bad bucket, shouldn't be here anyway. Test WILL fail" )
                
            if should_fail:
                self.fail("Should have caught exception for bad bucket name: " + bad_bucket)
        except:
            self.tester.debug( "Correctly caught the exception" )
        
        try:
            bad_bucket = self.bucket_prefix + "bucket*123"
            self.tester.create_bucket(bad_bucket)
            should_fail = True            
            try:
                self.tester.delete_bucket(bad_bucket)
            except:
                self.tester.debug( "Exception deleting bad bucket, shouldn't be here anyway. Test WILL fail" )
                
            if should_fail:
                self.fail("Should have caught exception for bad bucket name: " + bad_bucket)
        except:
            self.tester.debug( "Correctly caught the exception" )
        
        try:
            bad_bucket = self.bucket_prefix + "/bucket123"
            self.tester.create_bucket(bad_bucket)
            should_fail = True            
            try:
                self.tester.delete_bucket(bad_bucket)
            except:
                self.tester.debug( "Exception deleting bad bucket, shouldn't be here anyway. Test WILL fail" )
                
            if should_fail:
                self.fail("Should have caught exception for bad bucket name: " + bad_bucket)
        except:
            self.tester.debug( "Correctly caught the exception" )

        """
        Test creating bucket with null name
        """
        try:
            null_bucket_name = ""
            bucket_obj = self.tester.create_bucket(null_bucket_name)
            self.tester.sleep(10)
            if bucket_obj:
                self.fail("Should have caught exception for creating bucket with empty-string name.")
        except S3ResponseError as e:
            self.assertEqual(e.status, 405, 'Expected response status code to be 405, actual status code is ' + str(e.status))
            self.assertTrue(re.search("MethodNotAllowed", e.code), "Incorrect exception returned when creating bucket with null name.")
        except Exception, e:
            self.tester.debug("Failed due to EUCA-7059 " + str(e))

    def test_bucket_acl(self):
        '''
        Tests bucket ACL management and adding/removing from the ACL with both valid and invalid usernames
        '''
        
        test_bucket = self.bucket_prefix + 'acl_bucket_test'
        self.buckets_used.add(test_bucket)
        test_user_id = self.tester.s3.get_canonical_user_id()     
        self.tester.debug('Starting ACL test with bucket name: ' + test_bucket + ' and userid ' + test_user_id)
        
        try: 
            acl_bucket = self.tester.s3.create_bucket(test_bucket)
            self.tester.debug('Created bucket: ' + test_bucket)
        except S3CreateError:
            self.tester.debug( "Can't create the bucket, already exists. Deleting it an trying again" )
            try :
                self.tester.s3.delete_bucket(test_bucket)            
                acl_bucket = self.tester.s3.create_bucket(test_bucket)
            except:
                self.tester.debug( "Couldn't delete and create new bucket. Failing test" )
                self.fail("Couldn't make the test bucket: " + test_bucket)
                                
        policy = acl_bucket.get_acl()
        
        if policy == None:
            self.fail('No acl returned')
        
        self.tester.debug( policy )
        #Check that the acl is correct: owner full control.
        if len(policy.acl.grants) > 1:
            self.tester.s3.delete_bucket(test_bucket)
            self.fail('Expected only 1 grant in acl. Found: ' + policy.acl.grants.grants.__len__())

        if policy.acl.grants[0].id != test_user_id or policy.acl.grants[0].permission != 'FULL_CONTROL':
            self.tester.s3.delete_bucket(test_bucket)
            self.fail('Unexpected grant encountered: ' + policy.acl.grants[0].display_name + ' ' + policy.acl.grants[0].permission + ' ' + policy.acl.grants[0].id)

        #Get the info on the owner from the ACL returned
        owner_display_name = policy.acl.grants[0].display_name
        owner_id = policy.acl.grants[0].id
                                    
        #upload a new acl for the bucket
        new_acl = policy
        new_user_display_name = owner_display_name
        new_user_id = owner_id
        new_acl.acl.add_user_grant(permission="READ", user_id=new_user_id, display_name=new_user_display_name)
        try:
            acl_bucket.set_acl(new_acl)
            acl_check = acl_bucket.get_acl()
        except S3ResponseError:
            self.fail("Failed to set or get new acl")
        
        self.tester.info( "Got ACL: " + acl_check.acl.to_xml() )

        #expected_result_base='<AccessControlList>
        #<Grant><Grantee xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:type="CanonicalUser">
        #<ID>' + owner_id + '</ID><DisplayName>'+ owner_display_name + '</DisplayName></Grantee><Permission>FULL_CONTROL</Permission></Grant>
        #<Grant><Grantee xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:type="CanonicalUser"><ID>EXPECTED_ID</ID><DisplayName>EXPECTED_NAME</DisplayName></Grantee><Permission>READ</Permission></Grant>
        #</AccessControlList>'                
            
        if acl_check == None or not self.tester.check_acl_equivalence(acl1=acl_check.acl, acl2=new_acl.acl):
            self.tester.s3.delete_bucket(test_bucket)
            self.fail("Incorrect acl length or acl not found\n. Got bucket ACL:\n" + acl_check.acl.to_xml() + "\nExpected:" + new_acl.acl.to_xml())
        else:
            self.tester.info("Got expected basic ACL addition")
        
        self.tester.info( "Grants 0 and 1: " + acl_check.acl.grants[0].to_xml() + " -- " + acl_check.acl.grants[1].to_xml() )
        
        #Check each canned ACL string in boto to make sure Walrus does it right
        for acl in boto.s3.acl.CannedACLStrings:
            if acl == "authenticated-read":
                continue
            self.tester.info('Testing canned acl: ' + acl)
            try:
                acl_bucket.set_acl(acl)
                acl_check = acl_bucket.get_acl()
            except Exception as e:
                self.tester.s3.delete_bucket(test_bucket)
                self.fail("Got exception trying to set acl to " + acl + ": " + str(e))
            
            self.tester.info( "Set canned-ACL: " + acl + " -- Got ACL from service: " + acl_check.acl.to_xml() )            
            expected_acl = self.tester.get_canned_acl(bucket_owner_id=owner_id,canned_acl=acl, bucket_owner_display_name=owner_display_name)
                        
            if expected_acl == None:
                self.tester.s3.delete_bucket(test_bucket)
                self.fail("Got None when trying to generate expected acl for canned acl string: " + acl)
                        
            if not self.tester.check_acl_equivalence(acl1=expected_acl, acl2=acl_check.acl):
                self.tester.s3.delete_bucket(test_bucket)
                self.fail("Invalid " + acl + " acl returned from Walrus:\n" + acl_check.acl.to_xml() + "\nExpected\n" + expected_acl.to_xml())
            else:
                self.tester.debug( "Got correct acl for: " + acl  )        
        
        try:
            acl_bucket.set_acl('invalid-acl')
            self.fail('Did not catch expected exception for invalid canned-acl')            
        except:
            self.tester.debug( "Caught expected exception from invalid canned-acl" )        
        
        
        self.tester.s3.delete_bucket(test_bucket)
        self.buckets_used.remove(test_bucket)
        self.tester.debug( "Bucket ACL: PASSED"  )    
        pass    
    
    def test_bucket_key_list_delim_prefix(self):
        """Tests the prefix/delimiter functionality of key listings and parsing"""
        test_bucket_name = self.bucket_prefix + "testbucketdelim"
        self.buckets_used.add(test_bucket_name)
        self.tester.debug('Testing bucket key list delimiters and prefixes using bucket: ' + test_bucket_name)        
        try: 
            testbucket = self.tester.s3.create_bucket(bucket_name=test_bucket_name)
        except S3CreateError:
            self.tester.debug( "bucket already exists, using it" )
            try:
                testbucket = self.tester.s3.get_bucket(bucket_name=test_bucket_name)
            except S3ResponseError as err:
                self.tester.debug( "Fatal error: could to create or get bucket" )
                for b in self.tester.s3.get_all_buckets():
                    self.tester.debug( "Bucket: " + b.name   )             
                self.fail("Could not setup bucket, " + test_bucket_name + " for test: " + err.error_message )

        prefix = "users"
        delim = "/"
        
        for i in range(10):
            tmp = str(i)
            self.tester.debug("adding keys iteration " + tmp)
            key = testbucket.new_key("testobject" + tmp)
            key.set_contents_from_string("adlsfjaoivajsdlajsdfiajsfdlkajsfkajdasd")
            
            key = testbucket.new_key(prefix + "testkey" + tmp)
            key.set_contents_from_string("asjaoidjfafdjaoivnw")
            
            key = testbucket.new_key(prefix + delim + "object" + tmp)
            key.set_contents_from_string("avjaosvdafajsfd;lkaj")
            
            key = testbucket.new_key(prefix + delim + "objects" + delim + "photo" + tmp + ".jpg")
            key.set_contents_from_string("aoiavsvjasldfjadfiajss")
    
        keys = testbucket.get_all_keys(prefix=prefix, delimiter=delim, max_keys=10)
        self.tester.debug( "Prefix with 10 keys max returned: " + str(len(keys)) + " results" )
        
        for k in keys:
                self.tester.debug( k )
            
        keys = testbucket.get_all_keys(prefix=prefix, delimiter=delim, max_keys=20)
        self.tester.debug( "Prefix with 20 keys max returned: " + str(len(keys)) + " results" )
        
        for k in keys:
                self.tester.debug( k )
            
        print "Cleaning up the bucket"
        for i in range(10):
            testbucket.delete_key("testobject" + str(i))
            testbucket.delete_key(prefix + "testkey" + str(i))
            testbucket.delete_key(prefix + delim + "object" + str(i))
            testbucket.delete_key(prefix + delim + "objects" + delim + "photo" + str(i) + ".jpg")

        print "Deleting the bucket"
        self.tester.s3.delete_bucket(testbucket)
        
        
    def test_bucket_location(self):        
        test_bucket = self.bucket_prefix + "location_test_bucket"
        self.tester.info('Starting bucket location test using bucket: ' + test_bucket)
        self.tester.s3.create_bucket(test_bucket)
        
        bucket = self.tester.s3.get_bucket(test_bucket)
        if bucket != None and (bucket.get_location() == Location.DEFAULT or bucket.get_location() == 'US'):
            self.tester.s3.delete_bucket(test_bucket)
        else:
            bucket.delete()
            self.fail("Bucket location test failed, could not get bucket or location is not 'US'")        
        
        test_bucket = self.bucket_prefix + "eu_location_test"
        bucket = self.tester.s3.create_bucket(test_bucket,location=Location.EU)
        self.buckets_used.add(test_bucket)
        if bucket == None:
            self.fail("Bucket creation at location EU failed")
        else:
            loc = bucket.get_location()
            
        if loc == Location.EU:
            print "Got correct bucket location, EU"
            bucket.delete()
        else:                        
            print "Incorrect bucket location, failing"
            bucket.delete()
            self.fail("Bucket location incorrect, expected: EU, got: " + loc)
        
        self.buckets_used.remove(test_bucket)
        pass
        
    def test_bucket_logging(self):
        """This is not a valid test at the moment, logging requires at least an hour of time between logging enabled and file delivery of events to the dest bucket"""
        self.tester.info("\n\nStarting bucket logging test")
        test_bucket = self.bucket_prefix + "logging_test_bucket"
        log_dest_bucket = self.bucket_prefix + "logging_destination_test_bucket"
        self.buckets_used.add(test_bucket)
        self.buckets_used.add(log_dest_bucket)
        
        log_prefix = "log_prefix_test"

        try:
            bucket = self.tester.s3.create_bucket(test_bucket)
        except S3CreateError:
            self.tester.info("Bucket exists, trying to delete and re-create")
            try:
                self.tester.s3.delete_bucket(test_bucket)
                bucket = self.tester.s3.create_bucket(test_bucket)
            except:
                self.tester.info("Couldn't delete and create new bucket...failing")
                self.fail("Couldn't get clean bucket already existed and could not be deleted")
        
        try:        
            dest_bucket = self.tester.s3.create_bucket(log_dest_bucket)
        except S3CreateError:
            self.tester.info("Bucket exists, trying to delete and re-create")
            try:
                self.tester.s3.delete_bucket(log_dest_bucket)
                dest_bucket = self.tester.s3.create_bucket(log_dest_bucket)
            except:
                self.tester.info("Couldn't delete and create new bucket...failing")
                self.fail("Couldn't get clean bucket already existed and could not be deleted")
        
        log_delivery_policy = dest_bucket.get_acl()
        log_delivery_policy.acl.add_grant(Grant(type="Group",uri="http://acs.amazonaws.com/groups/s3/LogDelivery",permission="WRITE"))
        log_delivery_policy.acl.add_grant(Grant(type="Group",uri="http://acs.amazonaws.com/groups/s3/LogDelivery",permission="READ_ACP"))        
        dest_bucket.set_acl(log_delivery_policy)
        bucket.enable_logging(log_dest_bucket,target_prefix=log_prefix)        
        
        #test the logging by doing something that will require logging
        k = bucket.new_key('key1')
        k.set_contents_from_string('content123')
        
        k = bucket.new_key('key2')
        k.set_contents_from_string("content456")
        
        k = bucket.get_key('key1')
        result1 = k.get_contents_as_string()
        self.tester.info("Got content:\n\t " + result1)
        
        k = bucket.get_key('key2')
        result2 = k.get_contents_as_string()
        self.tester.info("Got content:\n\t " + result2)
        
        keylist = bucket.list()
        self.tester.info("Listing keys...")
        for k in keylist:
            if isinstance(k, boto.s3.prefix.Prefix):
                self.tester.info("Prefix found")
            else:
                self.tester.info('Key--' + k.name)
                
        #Allow some time for the op writes to be logged... this may need to be tweaked
        time.sleep(15)
        
        #Now check the log to be sure something was logged
        log_bucket = self.tester.s3.get_bucket(log_dest_bucket)
        for k in log_bucket.list(prefix=log_prefix):
            self.tester.info('Key -- ' + k.name)
            log_obj = log_bucket.get_key(k)
            self.tester.info("Log content:\n\t" + k.get_contents_as_string())
        
        log_data = log_obj.get_content_as_string()
        self.tester.info('Log data is: ' + log_data)

        self.tester.info('Deleting used bucket')
                
    def test_bucket_versioning(self):
        test_bucket = self.bucket_prefix + "versioning_test_bucket"
        self.tester.info('Testing bucket versioning using bucket:' + test_bucket)
        version_bucket = self.tester.s3.create_bucket(test_bucket)
        self.buckets_used.add(version_bucket)
        version_status = version_bucket.get_versioning_status().get("Versioning")
        
        #Test the default setup after bucket creation. Should be disabled.
        if version_status != None:
            version_bucket.delete()            
            self.fail("Expected versioning disabled, found: " + str(version_status))
        elif version_status == None:
            self.tester.info("Null version status returned, correct since it should be disabled")
        
        #Turn on versioning, confirm that it is 'Enabled'
        version_bucket.configure_versioning(True)        
        version_status = version_bucket.get_versioning_status().get("Versioning")
        if version_status == None or version_status != "Enabled":
            version_bucket.delete()
            self.fail("Expected versioning enabled, found: " + str(version_status))
        elif version_status == None:
            version_bucket.delete()
            self.fail("Null version status returned")    
        self.tester.info("Versioning of bucket is set to: " + version_status)
        
        #Turn off/suspend versioning, confirm.
        version_bucket.configure_versioning(False)
        version_status = version_bucket.get_versioning_status().get("Versioning")
        if version_status == None or version_status != "Suspended":
            version_bucket.delete()
            self.fail("Expected versioning suspended, found: " + str(version_status))
        elif version_status == None:
            version_bucket.delete()
            self.fail("Null version status returned")    
        
        self.tester.info("Versioning of bucket is set to: " + version_status)        
        
        version_bucket.configure_versioning(True)
        version_status = version_bucket.get_versioning_status().get("Versioning")
        if version_status == None or version_status != "Enabled":
            version_bucket.delete()
            self.fail("Expected versioning enabled, found: " + str(version_status))    
        elif version_status == None:
            version_bucket.delete()
            self.fail("Null version status returned")    
        
        self.tester.info("Versioning of bucket is set to: " + version_status)
        
        version_bucket.delete()
        self.buckets_used.remove(version_bucket)
        self.tester.info("Bucket Versioning: PASSED")
               
    def test_bucket_key_listing_paging(self):
        """Test paging of long results lists correctly and in alpha order"""        
        test_bucket_name = self.bucket_prefix + "pagetestbucket"
        self.buckets_used.add(test_bucket_name)
        self.tester.info('Testing bucket key listing pagination using bucket: ' + test_bucket_name)
        
        try:
            testbucket = self.tester.s3.create_bucket(bucket_name=test_bucket_name)
        except S3CreateError:
            self.tester.info("Bucket already exists, getting it")
            try:
                testbucket = self.tester.s3.get_bucket(bucket_name=test_bucket_name)
            except S3ResponseError as err:
                self.tester.info("Fatal error: could not get bucket or create it")
                for b in self.tester.s3.get_all_buckets():
                    self.tester.info('Bucket -- ' + b.name )                   
                self.fail("Could not get bucket, " + test_bucket_name + " to start test: " + err.error_message)                       
                        
        key_name_prefix = "testkey"
        
        for i in range(100):
            key_name = key_name_prefix + str(i)
            self.tester.info("creating object: " + key_name)
            testbucket.new_key(key_name).set_contents_from_string("testcontents123testtesttesttest")
            
        for i in range(100):
            key_name = key_name_prefix + "/key" + str(i)
            self.tester.info("creating object: " + key_name)
            testbucket.new_key(key_name).set_contents_from_string("testafadsoavlsdfoaifafdslafajfaofdjasfd")
                
        key_list = testbucket.get_all_keys(max_keys=50)        
        self.tester.info("Got " + str(len(key_list)) + " entries back")
        
        for k in key_list:
            assert isinstance(k, Key)
            self.tester.info('Found key -- ' + k.name)
        
        if len(key_list) != 50:
            self.fail("Expected 50 keys back, got " + str(len(key_list)))
            

        
        for i in range(100):
            key_name = key_name_prefix + str(i)
            self.tester.info("Deleting key: " + key_name)
            testbucket.delete_key(key_name)
            key_name = key_name_prefix + "/key" + str(i)
            self.tester.info("Deleting key: " + key_name)
            testbucket.delete_key(key_name)
                                
        self.tester.info("Cleaning up the bucket")
        
        key_list = testbucket.get_all_keys()
        
        for k in key_list:
            self.tester.info("Deleting key: " + k.key())
            testbucket.delete_key(k)

        self.tester.info("Deleting the bucket")
        self.tester.s3.delete_bucket(testbucket)
        self.buckets_used.add(test_bucket_name)
        
    def test_list_multipart_uploads(self):
        self.fail("Feature Not implemented")

    def test_bucket_lifecycle(self):        
        self.fail("Feature Not implemented")
        
    def test_bucket_policy(self):
        self.fail("Feature Not implemented")
        
    def test_bucket_website(self):
        self.fail("FeatureNot implemented")
    
    def clean_method(self):
        '''This is the teardown method'''
        #Delete the testing bucket if it is left-over
        self.tester.info('Deleting the buckets used for testing')
        for bucket in self.buckets_used:
            try:
                self.tester.info('Checking bucket ' + bucket + ' for possible cleaning/delete')
                if self.tester.s3.bucket_exists(bucket):
                    self.tester.info('Found bucket exists, cleaning it')
                    self.tester.clear_bucket(bucket)
                    self.buckets_used.remove(bucket)
                else:
                    self.tester.info('Bucket ' + bucket + ' not found, skipping')
            except:
                self.tester.info('Exception checking bucket ' + bucket)

        return
          
if __name__ == "__main__":
    testcase = BucketTestSuite()
    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    list = testcase.args.tests or [ 'test_bucket_get_put_delete', \
                                   #'test_bucket_acl', \ FAILING AS OF 3.3.1
                                   'test_bucket_key_list_delim_prefix', \
                                   'test_bucket_key_listing_paging', \
                                   'test_bucket_location', \
                                   'test_bucket_versioning']
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )
    ### Run the EutesterUnitTest objects

    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)


#!/usr/bin/env python

#
####################################
#                                  #
#   Walrus/S3 Object Test Cases    #
#                                  #
####################################


#Author: Zach Hill <zach@eucalyptus.com>
#Author: Vic Iglesias <vic@eucalyptus.com>

import time
import random

from boto.s3.key import Key
from boto.s3.prefix import Prefix
from boto.exception import S3ResponseError
import dateutil.parser

from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase
from eucaops import S3ops


class ObjectTestSuite(EutesterTestCase):
    data_size = 1000
    
    def __init__(self):
        self.setuptestcase()
        self.setup_parser()
        self.parser.add_argument("--s3endpoint", default=None)
        self.get_args()
        # Setup basic eutester object
        if self.args.s3endpoint:
            self.tester = S3ops( credpath=self.args.credpath, endpoint=self.args.endpoint)
        else:
            self.tester = Eucaops( credpath=self.args.credpath, config_file=self.args.config, password=self.args.password)
        
        self.bucket_prefix = "eutester-bucket-test-suite-" + str(int(time.time())) + "-"
        self.buckets_used = set()
        random.seed(time.time())
        self.test_bucket_name = self.bucket_prefix + str(random.randint(0,100))
        self.test_bucket = self.tester.create_bucket(self.test_bucket_name)
        self.buckets_used.add(self.test_bucket_name)
        #Create some test data for the objects
        self.test_object_data = ""
        for i in range(0, self.data_size):
            self.test_object_data += chr(random.randint(32,126))            
        print "Generated data for objects: " + self.test_object_data
        
    
    def print_key_info(self, keys=None):
        for key in keys:
            self.tester.info("Key=" + str(key.key) + " -- version= " + str(key.version_id) + " -- eTag= " + str(key.etag)
                             + " -- ACL= " + str(key.get_xml_acl()))
    
    def put_object(self, bucket=None, object_key=None, object_data=None):
        """Puts an object with the specified name and data in the specified bucket"""
        if bucket == None:
            raise Exception("Cannot put object without proper bucket reference")
        
        try :
            key = Key(bucket=bucket,name=object_key)
            key.set_contents_from_string(object_data)                        
            return key.etag
        except Exception as e:
            self.tester.info("Exception occured during 'PUT' of object " + object_key + " into bucket " + bucket.name + ": " + e.message)
            return None
        
     
    def enable_versioning(self, bucket):
        """Enable versioning on the bucket, checking that it is not already enabled and that the operation succeeds."""
        vstatus = bucket.get_versioning_status()
        if vstatus != None and len(vstatus.keys()) > 0 and vstatus['Versioning'] != None and vstatus['Versioning'] != 'Disabled':
            self.tester.info("Versioning status should be null/Disabled, found: " + vstatus['Versioning'])
            return False
        else:
            self.tester.info("Bucket versioning is Disabled")
        
        #Enable versioning
        bucket.configure_versioning(True)
        if bucket.get_versioning_status()['Versioning'] == 'Enabled':
            self.tester.info("Versioning status correctly set to enabled")
            return True
        else:
            self.tester.info("Versioning status not enabled, should be.")
            return False
        return False
    
    def suspend_versioning(self, bucket):
        """Suspend versioning on the bucket, checking that it is previously enabled and that the operation succeeds."""
        if bucket.get_versioning_status()['Versioning'] == 'Enabled':
            self.tester.info("Versioning status correctly set to enabled")
        else:
            self.tester.info("Versioning status not enabled, should be. Can't suspend if not enabled....")
            return False
    
        #Enable versioning
        bucket.configure_versioning(False)
        if bucket.get_versioning_status()['Versioning'] == 'Suspended':
            self.tester.info("Versioning status correctly set to suspended")
            return True
        else:
            self.tester.info("Versioning status not suspended.")
            return False
        return False 
             
    def check_version_listing(self, version_list, total_expected_length):
        """Checks a version listing for both completeness and ordering as well as pagination if required"""
        self.tester.info("Checking bucket version listing. Listing is " + str(len(version_list)) + " entries long")
        if total_expected_length >= 1000:
            assert(len(version_list) == 999)
        else:
            assert(len(version_list) == total_expected_length)
        
        prev_obj = None
        should_fail = None
        for obj in version_list:
            if isinstance(obj,Key):
                self.tester.info("Key: " + obj.name + " -- " + obj.version_id + "--" + obj.last_modified)                
                if prev_obj != None:
                    if self.compare_versions(prev_obj, obj) > 0:
                        should_fail = obj
                prev_obj = obj
            else:
                self.tester.info("Not a key, skipping: " + str(obj))
        return should_fail

    def compare_versions(self, key1, key2):
        """
        Returns -1 if key1 < key2, 0 if equal, and 1 if key1 > key2. 
        Compares names lexicographically, if equal, compares date_modified if versions are different. 
        If version_id and name are equal then key1 = key2
        If an error occurs or something is wrong, returns None
        """
        if key1.name < key2.name:
            #self.debug("Key1: " + key1.name + " is less than " + key2.name)
            return 1
        elif key1.name > key2.name:
            #self.debug("Key1: " + key1.name + " is greater than " + key2.name)
            return -1
        else:
            if key1.version_id == key2.version_id:
                #self.debug("Key1: " + key1.name + " is the same version as " + key2.name)
                return 0
            else:
                if dateutil.parser.parse(key1.last_modified) > dateutil.parser.parse(key2.last_modified):
                    #self.debug("Key1: " + key1.last_modified + " last modified is greater than " + key2.last_modified)
                    return 1
                elif dateutil.parser.parse(key1.last_modified) < dateutil.parser.parse(key2.last_modified):
                    #self.debug("Key1: " + key1.last_modified + " last modified is less than " + key2.last_modified)
                    return -1
        return None
    
    def test_object_basic_ops(self):
        """
        Tests basic operations on objects: simple GET,PUT,HEAD,DELETE.
        
        """
        self.tester.info("Basic Object Operations Test (GET/PUT/HEAD)")
        if self.test_bucket == None:
            self.fail("Error: test_bucket not set, cannot run test")
            
        #Test PUT & GET
        testkey="testkey1-" + str(int(time.time()))
        self.put_object(bucket=self.test_bucket, object_key=testkey, object_data=self.test_object_data)
        
        ret_key = self.test_bucket.get_key(testkey)
        ret_content = ret_key.get_contents_as_string()
        
        if ret_content == self.test_object_data:
            self.tester.info("Set content = get content, put passed")
        else:
            if ret_content != None:
                self.tester.info("Got content: " + ret_content)
            else:
                self.tester.info("No content returned")
            self.tester.info("Expected content: " + self.test_object_data)
            self.fail("Put content not the same as what was returned")
        
        #Test HEAD
        key_meta = self.test_bucket.get_key(testkey)
        if key_meta != ret_key:
            self.tester.info("Something is wrong, the HEAD operation returned different metadata than the GET operation")
        else:
            self.tester.info("HEAD meta = GET meta, all is good")
        
        #Test copy operation (GET w/source headers)
        new_key = "testkey2"
        self.test_bucket.copy_key(new_key, self.test_bucket_name,testkey)
        keylist = self.test_bucket.list()
        counter = 0
        for k in keylist:
            if isinstance(k, Prefix):
                self.tester.info("Prefix: " + "NULL" if k == None else k.name)
            else:
                self.tester.info("Key: " + k.name + " Etag: " + k.etag)
                counter += 1
        if counter != 2:
            self.fail("Expected 2 keys after copy operation, found only: " + len(keylist))
        try:
            ret_key = self.test_bucket.get_key(new_key)
        except:
            self.fail("Could not get object copy")
        if ret_key == None:
            self.fail("Could not get object copy")
            
        if self.test_bucket.get_key(testkey).get_contents_as_string() != ret_key.get_contents_as_string():
            self.fail("Contents of original key and copy don't match")
        else:
            self.tester.info("Copy key contents match original!")
        
        #Test DELETE
        self.test_bucket.delete_key(testkey)
        ret_key = None
        ret_key = self.test_bucket.get_key(testkey)
        if ret_key:
            self.tester.info("Erroneously got: " + ret_key.name)
            raise S3ResponseError(404, "Should have thrown exception for getting a non-existent object")
        self.tester.info("Finishing basic ops test")
               
    def test_object_byte_offset_read(self):
        """Tests fetching specific byte offsets of the object"""
        self.tester.info("Byte-range Offset GET Test")
        self.test_bucket = self.clear_and_rebuild_bucket(self.test_bucket_name)
        testkey = "rangetestkey-" + str(int(time.time()))
        source_bytes = bytearray(self.test_object_data)
        
        #Put the object initially
        self.put_object(bucket=self.test_bucket, object_key=testkey, object_data=self.test_object_data)
        
        #Test range for first 100 bytes of object
        print "Trying start-range object get"
        try:
            data_str = Key(bucket=self.test_bucket,name=testkey).get_contents_as_string(headers={"Range":"bytes=0-99"})
        except:
            self.fail("Failed range object get first 100 bytes")
        
        startrangedata = bytearray(data_str)        
        print "Got: " + startrangedata
        print "Expected: " + str(source_bytes[:100])
        start = 0        
        for i in range(0,100):
            if startrangedata[i-start] != source_bytes[i]:
                print "Byte: " + startrangedata[i] + " differs!"
                self.fail("Start-range Ranged-get failed")
            
        print "Trying mid-object range"   
        try: 
            data_str = Key(bucket=self.test_bucket,name=testkey).get_contents_as_string(headers={"Range":"bytes=500-599"})
        except:
            self.fail("Failed range object get for middle 100 bytes")     
        midrangedata = bytearray(data_str)
        start = 500
        for i in range(start,start+100):
            if midrangedata[i-start] != source_bytes[i]:
                print "Byte: " + midrangedata[i] + "differs!"
                self.fail("Mid-range Ranged-get failed")
        
        print "Trying end-range object get"
        #Test range for last 100 bytes of object
        try:
            data_str = Key(bucket=self.test_bucket,name=testkey).get_contents_as_string(headers={"Range":"bytes=800-899"})
        except:
            self.fail("Failed range object get for last 100 bytes")
            
        endrangedata = bytearray(data_str)
        print "Got: " + str(endrangedata)
        start = 800
        try:
            for i in range(start,start+100):
                if endrangedata[i-start] != source_bytes[i]:
                    print "Byte: " + endrangedata[i] + "differs!"
                    self.fail("End-range Ranged-get failed")
        except Exception as e:
            print "Exception! Received: " + e
        
        print "Range test complete"
        
    def test_object_post(self):
        """Test the POST method for putting objects, requires a pre-signed upload policy and url"""
        self.fail("Test not implemented")
                
    def test_object_large_objects(self):
        """Test operations on large objects (>1MB), but not so large that we must use the multi-part upload interface"""
        self.tester.info("Testing large-ish objects over 1MB in size on bucket" + self.test_bucket_name)
        self.test_bucket = self.clear_and_rebuild_bucket(self.test_bucket_name)
        test_data = ""
        large_obj_size_bytes = 5 * 1024 * 1024 #5MB
        self.tester.info("Generating " + str(large_obj_size_bytes) + " bytes of data")

        #Create some test data
        for i in range(0, large_obj_size_bytes):
            test_data += chr(random.randint(32,126))

        self.tester.info("Uploading object content of size: " + str(large_obj_size_bytes) + " bytes")        
        keyname = "largeobj-" + str(int(time.time()))
        self.put_object(bucket=self.test_bucket, object_key=keyname, object_data=test_data)
        self.tester.info("Done uploading object")

        ret_key = self.test_bucket.get_key(keyname)
        ret_data = ret_key.get_contents_as_string()
        
        if ret_data != test_data:
            self.fail("Fetched data and generated data don't match")
        else:
            self.tester.info("Data matches!")
        
        self.tester.info("Removing large object")
        self.test_bucket.delete_key(ret_key)
        self.tester.info("Complete large object test")
        pass
            
    def test_object_multipart(self):
        """Test the multipart upload interface"""
        self.fail("Feature not implemented")
        
    def test_object_versioning_enabled(self):
        """Tests object versioning for get/put/delete on a versioned bucket"""
        self.tester.info("Testing bucket Versioning-Enabled")
        self.test_bucket = self.clear_and_rebuild_bucket(self.test_bucket_name)
        if not self.enable_versioning(self.test_bucket):
            self.fail("Could not properly enable versioning")
             
        #Create some keys
        keyname = "versionkey-" + str(int(time.time()))
        
        #Multiple versions of the data
        v1data = self.test_object_data + "--version1"
        v2data = self.test_object_data + "--version2"
        v3data = self.test_object_data + "--version3"
        
        #Test sequence: put v1, get v1, put v2, put v3, get v3, delete v3, restore with v1 (copy), put v3 again, delete v2 explicitly
        self.put_object(bucket=self.test_bucket, object_key=keyname, object_data=v1data)
                
        #Get v1
        obj_v1 = self.test_bucket.get_key(keyname)
        self.tester.check_md5(eTag=obj_v1.etag,data=v1data)
        
        self.tester.info("Initial bucket state after object uploads without versioning:")
        self.print_key_info(keys=[obj_v1])
                
        #Put v2 (and get/head to confirm success)
        self.put_object(bucket=self.test_bucket, object_key=keyname,object_data=v2data)
        obj_v2 = self.test_bucket.get_key(keyname)
        self.tester.check_md5(eTag=obj_v2.etag,data=v2data)
        self.print_key_info(keys=[obj_v1, obj_v2])
        
        #Put v3 (and get/head to confirm success)
        self.put_object(bucket=self.test_bucket, object_key=keyname,object_data=v3data)
        obj_v3 = self.test_bucket.get_key(keyname)
        self.tester.check_md5(eTag=obj_v3.etag,data=v3data)
        self.print_key_info(keys=[obj_v1, obj_v2, obj_v3])
        
        #Get a specific version, v1
        v1_return = self.test_bucket.get_key(key_name=keyname,version_id=obj_v1.version_id)
        self.print_key_info(keys=[v1_return])
        
        #Delete current latest version (v3)
        self.test_bucket.delete_key(keyname)

        del_obj = self.test_bucket.get_key(keyname)
        if del_obj:
            self.tester.info("Erroneously got: " + del_obj.name)
            raise S3ResponseError(404, "Should have thrown this exception for getting a non-existent object")
        
        #Restore v1 using copy
        try:
            self.test_bucket.copy_key(new_key_name=obj_v1.key,src_bucket_name=self.test_bucket_name,src_key_name=keyname,src_version_id=obj_v1.version_id)
        except S3ResponseError as e:
            self.fail("Failed to restore key from previous version using copy got error: " + str(e.status))
            
        restored_obj = self.test_bucket.get_key(keyname)
        self.tester.check_md5(eTag=restored_obj.etag,data=v1data)
        self.print_key_info(keys=[restored_obj])
        
        #Put v3 again
        self.put_object(bucket=self.test_bucket, object_key=keyname,object_data=v3data)
        self.tester.check_md5(eTag=obj_v3.etag,data=v3data)
        self.print_key_info([self.test_bucket.get_key(keyname)])

        #Delete v2 explicitly
        self.test_bucket.delete_key(key_name=obj_v2.key,version_id=obj_v2.version_id)
        del_obj = self.test_bucket.get_key(keyname,version_id=obj_v2.version_id)
        if del_obj:
            raise S3ResponseError("Should have gotten 404 not-found error, but got: " + del_obj.key + " instead",404)

        #Show what's on top
        top_obj = self.test_bucket.get_key(keyname)
        self.print_key_info([top_obj])
        self.tester.check_md5(eTag=top_obj.etag,data=v3data)
        
        self.tester.info("Finished the versioning enabled test. Success!!")

    def clear_and_rebuild_bucket(self, bucket_name):
        self.tester.clear_bucket(bucket_name)
        return self.tester.create_bucket(bucket_name)

    def test_object_versionlisting(self):
        """
        Tests object version listing from a bucket
        """
        version_max = 3
        keyrange = 20
        self.tester.info("Testing listing versions in a bucket and pagination using " + str(keyrange) + " keys with " + str(version_max) + " versions per key")
        self.test_bucket = self.clear_and_rebuild_bucket(self.test_bucket_name)
        if not self.enable_versioning(self.test_bucket):
            self.fail("Could not enable versioning properly. Failing")
        
        key = "testkey-" + str(int(time.time()))
        keys = [ key + str(k) for k in range(0,keyrange)]        
        contents = [ self.test_object_data + "--v" + str(v) for v in range(0,version_max)]        

        try:
            for keyname in keys:
                #Put version_max versions of each key
                for v in range(0,version_max):
                    self.tester.info("Putting: " + keyname + " version " + str(v))
                    self.test_bucket.new_key(keyname).set_contents_from_string(contents[v])
        except S3ResponseError as e:
            self.fail("Failed putting object versions for test: " + str(e.status))
        listing = self.test_bucket.get_all_versions()
        self.tester.info("Bucket version listing is " + str(len(listing)) + " entries long")
        if keyrange * version_max >= 1000:
            if not len(listing) == 999:
                self.test_bucket.configure_versioning(False)
                self.tester.debug(str(listing))
                raise Exception("Bucket version listing did not limit the response to 999. Instead: " + str(len(listing)))
        else:
            if not len(listing) == keyrange * version_max:
                self.test_bucket.configure_versioning(False)
                self.tester.debug(str(listing))
                raise Exception("Bucket version listing did not equal the number uploaded. Instead: " + str(len(listing)))
        
        prev_obj = None
        for obj in listing:
            if isinstance(obj,Key):
                self.tester.info("Key: " + obj.name + " -- " + obj.version_id + "--" + obj.last_modified)                
                if prev_obj != None:
                    if self.compare_versions(prev_obj, obj) <= 0:
                        raise Exception("Version listing not sorted correctly, offending key: " + obj.name + " version: " + obj.version_id + " date: " + obj.last_modified)
                prev_obj = obj
            else:
                self.tester.info("Not a key, skipping: " + str(obj))
    
    def test_object_versioning_suspended(self):
        """Tests object versioning on a suspended bucket, a more complicated test than the Enabled test"""
        self.tester.info("Testing bucket Versioning-Suspended")
        self.test_bucket = self.clear_and_rebuild_bucket(self.test_bucket_name)
        #Create some keys
        keyname1 = "versionkey1-" + str(int(time.time()))
        keyname2 = "versionkey2-" + str(int(time.time()))
        keyname3 = "versionkey3-" + str(int(time.time()))
        keyname4 = "versionkey4-" + str(int(time.time()))
        keyname5 = "versionkey5-" + str(int(time.time()))
        v1data = self.test_object_data + "--version1"
        v2data = self.test_object_data + "--version2"
        v3data = self.test_object_data + "--version3"
        
        vstatus = self.test_bucket.get_versioning_status()
        if vstatus:
            self.fail("Versioning status should be null/Disabled but was: " + str(vstatus))
        else:
            self.tester.info("Bucket versioning is Disabled")
        
        self.put_object(bucket=self.test_bucket, object_key=keyname1, object_data=v1data)
        self.put_object(bucket=self.test_bucket, object_key=keyname2, object_data=v1data)
        self.put_object(bucket=self.test_bucket, object_key=keyname3, object_data=v1data)
        self.put_object(bucket=self.test_bucket, object_key=keyname4, object_data=v1data)
        self.put_object(bucket=self.test_bucket, object_key=keyname5, object_data=v1data)
                    
        key1 = self.test_bucket.get_key(keyname1)        
        key2 = self.test_bucket.get_key(keyname2)        
        key3 = self.test_bucket.get_key(keyname3)        
        key4 = self.test_bucket.get_key(keyname4)        
        key5 = self.test_bucket.get_key(keyname5)

        self.tester.info("Initial bucket state after object uploads without versioning:")
        self.print_key_info(keys=[key1,key2,key3,key4,key5])
        
        
        
        #Enable versioning
        self.test_bucket.configure_versioning(True)
        if self.test_bucket.get_versioning_status():
            self.tester.info("Versioning status correctly set to enabled")
        else:
            self.tester.info("Versionign status not enabled, should be.")            
        
        #Update a subset of the keys
        key1_etag2=self.put_object(bucket=self.test_bucket, object_key=keyname1,object_data=v2data)
        key2_etag2=self.put_object(bucket=self.test_bucket, object_key=keyname2,object_data=v2data)
        
        key3_etag2=self.put_object(bucket=self.test_bucket, object_key=keyname3,object_data=v2data)
        key3_etag3=self.put_object(bucket=self.test_bucket, object_key=keyname3,object_data=v3data)
        
        #Delete a key
        self.test_bucket.delete_key(keyname5)

        #Suspend versioning
        self.test_bucket.configure_versioning(False)
        
        #Get latest of each key
        key1=self.test_bucket.get_key(keyname1)
        key2=self.test_bucket.get_key(keyname2)
        key3=self.test_bucket.get_key(keyname3)
        key4=self.test_bucket.get_key(keyname4)
        key5=self.test_bucket.get_key(keyname5)
        
        #Delete a key
        
        #Add a key
        
        #Add same key again
        
        #Fetch each key
    
    def test_object_acl(self):
        """Tests object acl get/set and manipulation"""
        self.fail("Test not implemented")
        
        #TODO: test custom and canned acls that are both valid an invalid
        
    def test_object_torrent(self):
        """Tests object torrents"""
        self.fail("Feature not implemented yet")

    
    def clean_method(self):
        '''This is the teardown method'''
        #Delete the testing bucket if it is left-over
        self.tester.info('Deleting the buckets used for testing')
        for bucket in self.buckets_used:
            try:
                self.tester.info('Checking bucket ' + bucket + ' for possible cleaning/delete')
                if self.tester.s3.bucket_exists(bucket):
                    self.tester.info('Found bucket exists, cleaning it')
                    self.tester.clear_bucket(bucket)
                    self.buckets_used.remove(bucket)
                else:
                    self.tester.info('Bucket ' + bucket + ' not found, skipping')
            except:
                self.tester.info('Exception checking bucket ' + bucket)

        return
          
if __name__ == "__main__":
    
    testcase = ObjectTestSuite()
    ### Either use the list of tests passed from config/command line to determine what subset of tests to run
    list = testcase.args.tests or ['test_object_basic_ops', \
                                   #'test_object_byte_offset_read', \
                                   'test_object_large_objects', \
                                   'test_object_versionlisting', \
                                   'test_object_versioning_enabled', \
                                   'test_object_versioning_suspended']                                   
    ### Convert test suite methods to EutesterUnitTest objects
    unit_list = [ ]
    for test in list:
        unit_list.append( testcase.create_testunit_by_name(test) )
    ### Run the EutesterUnitTest objects

    result = testcase.run_test_case_list(unit_list,clean_on_exit=True)
    exit(result)


# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: vic.iglesias@eucalyptus.com

import time

class RepoUtils:
    def __init__(self, machine, package_manager="yum" ):
        self.package_manager = None
        if package_manager is "yum":
            self.package_manager = Yum(machine)
        if package_manager is "apt":
            self.package_manager = Apt(machine)

class Package:
    name = None
    version = None
    

class PackageManager:
    name = None
    machine = None
    
    def install(self, package):
        raise NotImplementedError("Method not implemented for package manager " + str(self.name))
    
    def upgrade(self, package = None):
        raise NotImplementedError("Method not implemented for package manager " + str(self.name))
    
    def add_repo(self, url, name= None):
        raise NotImplementedError("Method not implemented for package manager " + str(self.name))
        
    def update_repos(self):
        raise NotImplementedError("Method not implemented for package manager " + str(self.name))
    
    def get_package_info(self):
        raise NotImplementedError("Method not implemented for package manager " + str(self.name))
    
    def get_installed_packages(self):
        raise NotImplementedError("Method not implemented for package manager " + str(self.name))
 
class Yum(PackageManager):
    def __init__(self, machine):
        self.machine = machine
        self.name = "yum"
        
    def install(self, package, nogpg=False):
        gpg_flag = ""
        if nogpg:
            gpg_flag = "--nogpg"

        self.machine.sys("yum install -y " + gpg_flag +  " " + package)
    
    def upgrade(self, package = None, nogpg=False):
        gpg_flag = ""
        if nogpg:
            gpg_flag = "--nogpg"
        if not package:
            package = ""
        self.machine.sys("yum upgrade -y " + gpg_flag +  " " + package, timeout=480)
    
    def add_repo(self, url, name= None):
        if name is None:
            name = "new-repo-" + str(int(time.time()))
        repo_file = "/etc/yum.repos.d/" + name + ".repo"
        self.machine.sys("echo '[%s]' > %s" % (name, repo_file) )
        self.machine.sys("echo 'name=%s' >> %s"  % (name, repo_file)  );
        self.machine.sys("echo 'baseurl=%s' >> %s" % (url, repo_file) )
        self.machine.sys("echo -e 'enabled=1\ngpgcheck=0' >> %s " % repo_file)
        self.update_repos()
        
    def update_repos(self):
        self.machine.sys("yum clean all")
        
class Apt(PackageManager):
    def __init__(self, machine):
        self.machine = machine
        self.name = "apt"
        self.apt_options = "-o Dpkg::Options::='--force-confold' -y --force-yes "
        
    def install(self, package):
        self.machine.sys("export DEBIAN_FRONTEND=noninteractive; apt-get install %s %s" % (self.apt_options, str(package)))
    
    def upgrade(self, package = None):
        if package is None:
            package = ""
        self.machine.sys("export DEBIAN_FRONTEND=noninteractive; apt-get dist-upgrade %s %s " % (self.apt_options, str(package)) )
    
    def add_repo(self, url, name= None):
        if name is None:
            name = "new-repo-" + str(int(time.time()))
        repo_file = "/etc/apt/sources.list.d/" + name
        self.machine.sys("echo %s >> %s " % (url, repo_file) )
        self.update_repos()
    
    def update_repos(self):
        self.machine.sys("apt-get update")
        
        
        
        


'''
Created on Nov 10, 2011

@author: clarkmatthew
'''
import time

class Webui_globals(object):
    def __init__(self):
        self.waitafterfail = 30
        
        #from main/start page
        #self.xUserLink='/html/body/div[3]/div[2]/div/div[2]/div/div[2]/div/div/table'
        self.xUserLink='/html/body/div[3]/div[2]/div/div[2]/div/div[2]/div/div/table/tbody/tr/td/a'
        
        
        #Left Nav links
        self.xLn_StartLink=     '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div/div[2]/div/div/div/div/div/div[2]'
        self.xLn_ServComp=      '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div/div[2]/div/div/div[2]/div/div/div[2]'
        self.xLn_IdMgmt=        '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div[2]/div/div/div[2]'
        self.xLn_Accounts =     '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div[2]/div[2]/div/div/div/div/div/div[2]'
        self.xLn_Groups=        '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div[2]/div[2]/div/div/div[2]/div/div/div[2]'
        self.xLn_Users=         '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div[2]/div[2]/div/div/div[3]/div/div/div[2]'
        self.xLn_Policies=      '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div[2]/div[2]/div/div/div[4]/div/div/div[2]'
        self.xLn_Keys=          '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div[2]/div[2]/div/div/div[5]/div/div/div[2]'
        self.xLn_Cert=          '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div[2]/div[2]/div/div/div[6]/div/div/div[2]'
        self.xLn_ResMgmt=       '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div[3]/div/div/div[2]'
        self.xLn_Images=        '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div[3]/div/div/div[2]'
        self.xLn_VmTypes=       '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div[3]/div[2]/div/div/div[2]/div/div/div[2]'
        self.xLn_UsageReport=   '/html/body/div[3]/div[2]/div/div[5]/div/div[3]/div/div/div/div/div/div/div[3]/div[2]/div/div/div[3]/div/div/div[2]'
        
        #login page links
        self.xSignUp= '/html/body/div[3]/div[2]/div/div[2]/div/a'
        self.AccountSignUpInvalidName ='invalid character for account name'
        self.AccountSignUpInvalidEmail = 'not look like a valid email address'
        self.AccountSignUpPassNoMatch = 'do not match'
        self.AccountSignUpPassTooShort = 'length must be at least'
        self.AccountSignUpSuccess="signup succeeded"
        self.AccountSignUpError = "Failed to complete the account signup"
        self.PasswordResetTxt = "reset request is sent"
        self.PasswordTxt = "Password"
        self.PasswordResetForm = "enter your new password"
        self.PasswordRestSuccess = 'password is successfully reset'
        self.FirstTimeLogin ='First time login'
        
        
        #general page locations
        self.xPage='//*'
        self.xLeftNav='/html/body/div[3]/div[2]/div/div[5]'    
        self.xRightNavTable = '/html/body/div[3]/div[2]/div/div[7]/div/div[4]/div/div/table'
        self.xRightNavTableRow1 = self.xRightNavTable+'/tbody/tr'
        self.xRighNavSaveButton = '/html/body/div[3]/div[2]/div/div[7]/div/div[3]/div/table/tbody/tr/td/button'
        self.xLogButton = '/html/body/div[3]/div[2]/div/div[3]/div/div[4]/div/a/span'
        self.xRow1LogMsg = '/html/body/div[3]/div[2]/div/div[4]/div/div/table/tbody/tr/td[3]'
        self.xLogMessage = '/html/body/div[3]/div[2]/div/div[3]/div/div/div'
        
        #Accounts Page           
        self.AccountsTopNavButtonNewAccount ='/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td'
        self.AccountsTopNavButtonDelAccount ='/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[2]'
        self.AccountsTopNavButtonNewUser=    '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[4]'
        self.AccountsTopNavButtonNewGroup= '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[5]'
        self.AccountsTopNavButtonAddPolicy = '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[7]'
        self.AccountsTopNavButtonApprove = '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[9]'
        self.AccountsTopNavButtonReject = '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[10]'
        #accounts table locations
        self.xAccountsTable='/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[3]/div/div[2]/div/div[3]/div/div/div/table'
        self.xAccountsRow1=self.xAccountsTable+'/tbody/tr'
        self.xAccountsRow1Id=self.xAccountsRow1+'/td'
        #accounts right nav locations
        self.xAccountsRightNavTable= self.xRightNavTable
        self.xAccountRightNavRow1=self.xAccountsRightNavTable+'/tbody/tr'
        self.xAccountForwardButton='/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[4]/img'
        #account page range is the current page / pages available between forward and reverse buttons
        self.xAccountPageRange ='/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[3]'
        self.AccountsRightNav_Name = 'Name'
        self.AccountsRightNav_RegStatus = 'Registration status'
        self.StatusRegistered = "REGISTERED"
        self.StatusConfirmed = "CONFIRMED"
        self.StatusApproved = "APPROVED"
        
        self.urlSearchAccountByName='#account:name='
        self.urlSearchAccountById='#account:id='
        
        
        #Users
        self.UsersTopNavButtonDeleteUser = '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td'
        self.UsersTopNavButtonAddToGroup = '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[3]'
        self.UsersTopNavButtonRemoveFromGroup = '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[4]'
        self.UsersTopNavButtonAddPolicy = '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[6]'
        self.UsersTopNavButtonAddKey = '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[7]'
        self.UsersTopNavButtonAddCert = '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[8]'
        self.UsersTopNavButtonApprove = '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[10]'
        self.USersTopNavButtonReject = '/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/table/tbody/tr/td[11]'
        self.xUsersTable='/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[3]/div/div[2]/div/div[3]/div/div/div/table'
        self.xUsersRow1 = self.xUsersTable+'/tbody/tr'   
        self.xUsersRow1Id= self.xUsersRow1+'/td' 
        self.urlSearchUserByName='#user:name='
        self.urlSearchUserByAccount='#user:account='
        self.xUsersRightNavTable = self.xRightNavTable
        self.xUsersRightNavRow1 = self.xRightNavTableRow1
        self.xUserEnabledTxt = "true"
        self.xUserDisabledTxt = "false"
        self.UsersRightNav_Enabled ='Enabled'
        self.UsersRighNav_Password = 'Password'
        #self.ApproveButton = '' Use click link by text...
        #self.RejectButton = ''
        
        #groups
        #self.xGroupRow1Id'/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[3]/div/div[2]/div/div[3]/div/div/div/table/tbody/tr/td'
        self.xGroupTable='/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[3]/div/div[2]/div/div[3]/div/div/div/table/'
        self.xGroupRow1=self.xGroupTable+'/tbody/tr'
        self.xGroupRow1Id=self.xGroupRow1+'/td'
        self.urlSearchGroupByName='#group:name='
        
        #vmtypes
        self.xVmTypeTable='/html/body/div[3]/div[2]/div/div[8]/div/div[3]/div/div[2]/div/div[2]/div/div[3]/div/div/div/table'
        self.xVmTypeRow1=self.xVmTypeTable+'/tbody/tr'
        self.xVmTypeRow1Id=self.xVmTypeRow1+'/td'
        self.vmTypeRightNavNameTxt="Name"
        self.vmTypeRightNavCpuTxt="CPUs"
        self.vmTypeRightNavMemTxt='Memory (MB)'
        self.vmTypeRightNavDiskTxt='Disk (GB)'
        self.urlSearchVmType='#vmtype:'
        
        #keys
        self.xaddKeyPopupKey= '/html/body/div[6]/div/table/tbody/tr[2]/td[2]/div/div/table/tbody/tr[2]/td/div/div/table/tbody/tr/td'
        self.xaddKeyPopupUser= '/html/body/div[6]/div/table/tbody/tr[2]/td[2]/div/div/table/tbody/tr[2]/td/div/div/table/tbody/tr/td[2]'
        self.xaddKeyOkButton='/html/body/div[6]/div/table/tbody/tr[2]/td[2]/div/div/table/tbody/tr[3]/td/a'
        self.urlSearchKeyID='#key:id='
        
        
        #Text used to verify pages have loaded...
        self.StartTxt='START GUIDE'
        self.ServCompTxt='SERVICE COMPONENTS'
        self.AccountsTxt='ACCOUNTS'
        self.GroupsTxt='GROUPS'
        self.UsersTxt='USERS'
        self.PoliciesTxt='ACCESS POLICIES'
        self.KeysTxt='ACCESS KEYS'
        self.CertTxt='X509 CERTIFICATES'
        self.ImagesTxt='VIRTUAL MACHINE IMAGES'
        self.VmTypesTxt='VIRTUAL MACHINE TYPES'
        self.UsageReportTxt='USAGE REPORT'
        self.rightNavTxt = 'PROPERTIES'
        
            
    def testCase(self,results, method, *args):
        name = method.__name__
        print "+++++++++++++++++++++++++++++++++++++++++++++++++++++++"
        self.printStarting()
        tstr = "TEST("+name+")"
        print "\n\n\nRunning " + tstr
        time.sleep(5)
        try: 
            method(*args)
            print  tstr+": PASSED"
            self.printPassed()
            results.append((tstr," PASSED", ""))
        except testCaseSkipped, se:
            reason = str(se)
            print tstr+": SKIPPED:" +reason
            self.printPassed()
            results.append((tstr," SKIPPED", ""))
        except Exception, e:
            reason = str(e)
            print tstr+": FAILED:" +reason
            self.printFailed()
            print "sleeping for ("+str(self.waitafterfail)+")"
            time.sleep(self.waitafterfail)
            results.append((tstr," FAILED", reason))
    
    def printResults(self,results):
        print "\n\n\n"
        for test in results:
            name,res,reason = test
            print '{0:55} ==> {1:6}:{2}'.format(name,res,reason)
            
    def printStarting(self):
        print "\n"
        print '''
         _              _   _              
        | |            | | (_)             
     ___| |_  __ _ _ __| |_ _ _ __   __ _  
    / __| __|/ _` | '__| __| | '_ \ / _` | 
    \__ \ |_| (_| | |  | |_| | | | | (_| | 
    |___/\__|\__,_|_|   \__|_|_| |_|\__, | 
                                     __/ | 
                                    |___/  
                                        
    '''
    def printClosing(self):
        print "\n"
        print ''' 
     _____ _           _ 
    /  __ \ |         (_)
    | /  \/ | ___  ___ _ _ __   __ _ 
    | |   | |/ _ \/ __| | '_ \ / _` |
    | \__/\ | (_) \__ \ | | | | (_| |
     \____/_|\___/|___/_|_| |_|\__, |
                                __/ |
                               |___/ 
    
    '''
        print "\n"
    
    
    
    def printPassed(self):
        print "\n"
        print ''' 
    ______  ___  _____ _____ ___________  
    | ___ \/ _ \/  ___/  ___|  ___|  _  \ 
    | |_/ / /_\ \ `--.\ `--.| |__ | | | | 
    |  __/|  _  |`--. \`--. \  __|| | | | 
    | |   | | | /\__/ /\__/ / |___| |/ /  
    \_|   \_| |_|____/\____/\____/|___/   
    
    '''
        print "\n"
    
    
    def printFailed(self):
        print "\n"
        print '''
    ______ ___  _____ _      ___________ 
    |  ___/ _ \|_   _| |    |  ___|  _  \ 
    | |_ / /_\ \ | | | |    | |__ | | | | 
    |  _||  _  | | | | |    |  __|| | | | 
    | |  | | | |_| |_| |____| |___| |/ /  
    \_|  \_| |_/\___/\_____/\____/|___/  
                                         
    '''
        print "\n"        
        
class testCaseSkipped(Exception):
    def __init__(self, value):
        self.value = value
    def __str__ (self):
        return repr(self.value)
    
    
    



#!/usr/bin/python
#
#
# Description:  This script encompasses test cases/modules concerning testing CanonicalizedAmzHeaders Element 
#               construction for Eucalyptus Walrus. (The HTTP status errors should match what happens on AWS S3)
#
##########################
#                        #
#       Test Cases       #
#                        #
##########################
#
# [CanonicalizedAmzHeaders]
#
#       This case was developed to see if HTTP status was sent back when doing a GET request 
#       for a listing of buckets with null CanonicalizedAmzHeaders.  
#
# [CanonicalizedAmzCommaHeaders]
#
#       This case was developed to see if HTTP status was sent back when doing a GET request 
#       for a listing of buckets with comma separated CanonicalizedAmzHeaders.  
#
# [CanonicalizedAmzSeparateHeaders]
#
#       This case was developed to see if HTTP status was sent back when doing a GET request 
#       for a listing of buckets with separated CanonicalizedAmzHeaders.  
# 
# [CanonicalizedAmzSeparateHeadersBroken]
#
#       This case was developed to see if HTTP status was sent back when doing a GET request 
#       for a listing of buckets with separated CanonicalizedAmzHeaders, but in a broken format.  
#

import unittest
import time
from eucaops import Eucaops
from eutester import xmlrunner
from rest import S3Connection, Auth
import os
import random
import argparse
import sys

class WalrusBasics(unittest.TestCase):

    def setUp(self):
        if options.credpath:
            self.tester = Eucaops(credpath=options.credpath)
        else:
            print "\tNeed to pass --credpath option. Try --help for more information\n"  
            exit(1)

        protocol = "http://"
        self.server = protocol + self.tester.s3.server_name() + self.tester.s3.path
        self.auth = Auth(self.tester.get_access_key(), self.tester.get_secret_key())
        """
        Create auth and attempt to authenticate to Walrus via S3Connection
        """
        try:
            self.connection = S3Connection(self.server, self.auth)
        except Exception, e:
            exit('Failed to establish authentication to Walrus due to: ' + str(e))

    def CanonicalizedAmzNullHeaders(self):
        self.tester.debug("###")
        self.tester.debug("\tStarting CanonicalizedAmzNullHeaders test")
        self.tester.debug("###")
        """
        Create null headers
        """
        test_headers = {}
        """
        Check to see if GET will work with null headers
        """
        status = self.connection.request("GET", "/", self.connection.fix_resource("/"), '', headers=test_headers)
        """
        status should return 200, with body in response
        """
        self.assertEqual(status[0], 200, 'Expected status code to be 200, actual status code is ' + str(status[0]))
        self.assertNotEqual(status[1], 0, 'Expected status length is greater than 0')

        self.tester.debug("###")
        self.tester.debug("\tSuccessfully completed CanonicalizedAmzNullHeaders test")
        self.tester.debug("###")

    def CanonicalizedAmzCommaHeaders(self):
        self.tester.debug("###")
        self.tester.debug("\tStarting CanonicalizedAmzCommaHeaders test")
        self.tester.debug("###")
        """
        Create x-amz-wutang comma headers
        """
        test_headers = {'x-amz-wutang': 'RZA,GZA,Ol Dirty Bastard,Method Man,Raekwon,Ghostface Killah,Inspectah Deck,U-God,Masta Killa'}
        """
        Check to see if GET will work with x-amz-wutang comma headers
        """
        status = self.connection.request("GET", "/", self.connection.fix_resource("/"), '', headers=test_headers)
        """
        status should return 200, with body in response
        """
        self.assertEqual(status[0], 200, 'Expected status code to be 200, actual status code is ' + str(status[0]))
        self.assertNotEqual(status[1], 0, 'Expected status length is greater than 0')

        self.tester.debug("###")
        self.tester.debug("\tSuccessfully completed CanonicalizedAmzCommaHeaders test")
        self.tester.debug("###")

    def CanonicalizedAmzSeparateHeaders(self):
        self.tester.debug("###")
        self.tester.debug("\tStarting CanonicalizedAmzSeparateHeaders test")
        self.tester.debug("###")
        """
        Create x-amz-wutang separated header 
        """
        test_headers = {'x-amz-wutang': ['RZA','GZA','Ol Dirty Bastard','Method Man','Raekwon','Ghostface Killah','Inspectah Deck','U-God','Masta Killa']}
        """
        Check to see if GET will work with x-amz-wutang separated header
        """
        status = self.connection.request("GET", "/", self.connection.fix_resource("/"), '', headers=test_headers)
        """
        status should return 200, with body in response
        """
        self.assertEqual(status[0], 200, 'Expected status code to be 200, actual status code is ' + str(status[0]))
        self.assertNotEqual(status[1], 0, 'Expected status length is greater than 0')

        self.tester.debug("###")
        self.tester.debug("\tSuccessfully completed CanonicalizedAmzSeparateHeaders test")
        self.tester.debug("###")

    def CanonicalizedAmzSeparateHeadersBroken(self):
        self.tester.debug("###")
        self.tester.debug("\tStarting CanonicalizedAmzSeparateHeadersBroken test")
        self.tester.debug("###")
        """
        Create x-amz-wutang separated headers broken up 
        """
        test_header1 = {'x-amz-wutang': ['RZA','GZA','Method Man','Raekwon','Ghostface Killah','Inspectah Deck','U-God','Masta Killa']}
        test_header2 = ['x-amz-wutang:RZA']
        """
        Check to see if GET will work with x-amz-wutang separated header broken up;
        should come back with 403 error complaining about mismatch signature
        """
        status = self.connection.request("GET", "/", self.connection.fix_resource("/"), '', headers=test_header1, x_amz_headerlist=test_header2)
        """
        status should return 403, with body in response
        """
        self.assertEqual(status[0], 403, 'Expected status code to be 403, actual status code is ' + str(status[0]))
        self.assertNotEqual(status[1], 0, 'Expected status length is greater than 0')

        self.tester.debug("###")
        self.tester.debug("\tSuccessfully completed CanonicalizedAmzSeparateHeadersBroken test")
        self.tester.debug("###")

    def tearDown(self):
        self.tester=None
        self.server=None
        self.auth=None
        self.connection=None

def get_options():
    ### Parse args
    ## If given command line arguments, use them as test names to launch
    parser = argparse.ArgumentParser(prog="canonical_headers_test.py", 
        description='Test Case [canonical_headers_test.py] Version 0.0.1 - Unit test for CanonicalizedAmzHeaders elements against Eucalyptus Walrus')
    parser.add_argument('--credpath', dest='credpath', required=True, help='Path to folder containing credentials')
    parser.add_argument('--xml', action="store_true", default=False)
    parser.add_argument('--tests', nargs='+', default= ["CanonicalizedAmzNullHeaders","CanonicalizedAmzCommaHeaders","CanonicalizedAmzSeparateHeaders","CanonicalizedAmzSeparateHeadersBroken"])
    parser.add_argument('unittest_args', nargs='*')

    ## Grab arguments passed via commandline
    options = parser.parse_args() 
    sys.argv[1:] = options.unittest_args
    return options

if __name__ == "__main__":
    ## If given command line arguments, use them as test names to launch
    options = get_options()
    for test in options.tests:
        if options.xml:
            file = open("test-" + test + "result.xml", "w")
            result = xmlrunner.XMLTestRunner(file).run(WalrusBasics(test))
        else:
            result = unittest.TextTestRunner(verbosity=2).run(WalrusBasics(test))
        if result.wasSuccessful():
            pass
        else:
            exit(1)


# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# author: clarkmatthew
# modified by: Trevor Hodde

'''
    Example:
    import eulogger
    self.logger = eulogger.Eulogger(name='euca')
    self.log = self.logger.log
    
    self.log.debug("This is a debug message")
    self.log.critical("this is a critical message")
'''

import os
import sys
import logging
import time

class Eulogger(object):
    #constructor for the Eulogger
    def __init__(self,
                 parent_logger_name = 'eutester',
                 identifier="eulogger",
                 stdout_level="debug",
                 stdout_format = None,
                 logfile = "",
                 logfile_level="debug",
                 make_log_file_global=True,
                 use_global_log_files=True,
                 file_format = None,
                 clear_file = False):
        """
        This class basically sets up a child debugger for testing purposes.
        It allows the user to set up a new logger object and pass different logging formats and levels so different
        objects and modules can log with unique identifiers and logging levels.


        :param parent_logger_name: Name of root/parent logger
        :param identifier: identifier used for log formatting and child logger name
        :param stdout_level: log level (see 'logging' class) for std out handler under this child logger
        :param stdout_format: logging format used by this child logger's stdout handler
        :param logfile: file path to use for this child logger's logging file handler
        :param logfile_level: log level (see 'logging' class) for file handler under this child logger
        :param file_format: logging formate used by this child logger's file handler
        :param clear_file: will attempt to remove 'logfile' before creating handler. Will not remove parent's files.
        :param make_log_file_global: boolean, will add this logfile to parent so other child loggers create afterward
                                     will attempt to create a handler that writes to this file as well.
        :param use_global_log_files: boolean, will query the parent logger for any file handlers and will attemp to
                                     create a handler for this child logger using the same file

        #Debug for init...
        print ( "-----------------------------------------------" \
                + "\nparent_logger_name:" + str(parent_logger_name) \
                + "\neulogger init:" \
                + "\nidentifier:" + str(identifier) \
                + "\nstdout_level:" + str(stdout_level) \
                + "\nstdout_format:" + str(stdout_format) \
                + "\nlogfile:" + str(logfile) \
                + "\nlogfile_level:" + str(logfile_level) \
                + "\nfile_format:" + str(file_format) \
                + "\nclear_file:" + str(clear_file) \
                + "\n-----------------------------------------------" )
        """
        self.logfile = os.path.join(logfile)
        self.clear_file = clear_file

        #Create of fetch existing logger of name 'logger_name
        self.parent_logger_name = parent_logger_name
        self.identifier = identifier
        self.name = identifier + str(time.time())
        self.parent_logger = logging.getLogger(self.parent_logger_name)
        self.log = self.getChild(self.parent_logger, self.name)
        self.file_info_list = []

        #map string for log level to 'logging' class type or default to logging.DEBUG if string isn't found
        self.stdout_level = logging.__dict__.get(stdout_level.upper(),logging.DEBUG)
        self.logfile_level = logging.__dict__.get(logfile_level.upper(),logging.DEBUG)

        #set the parent and child logger levels to the lowest of the two handler levels
        if self.stdout_level < self.logfile_level:
            self.logger_level = self.stdout_level
        else:
            self.logger_level = self.logfile_level
        if self.log.level > self.logger_level or self.log.level == 0:
            self.log.setLevel(self.logfile_level)
        if self.parent_logger > self.logger_level or self.log.level == 0:
            self.parent_logger.setLevel(self.logfile_level)


        #set some default and canned formatters for logging output
        self.default_format = stdout_format or logging.Formatter('[%(asctime)s] [' + self.identifier + '] [%(levelname)s]: %(message)s')
        self.file_format = file_format or self.default_format

        #Add a few canned formatters for reference/convenience
        self.formatter2 = logging.Formatter('[%(asctime)s] [' + self.identifier + '] [%(levelname)s] [%(filename)s:%(funcName)s():%(lineno)d]: %(message)s')
        self.formatter3 = logging.Formatter( self.identifier +':%(funcName)s():%(lineno)d: %(message)s')
        self.formatter4 = logging.Formatter('%(message)s')

        self.stdout_handler = logging.StreamHandler(sys.stdout)
        self.stdout_handler.setFormatter(self.default_format)
        self.stdout_handler.setLevel(self.stdout_level)
        #Add filter so only log records from this child logger are handled
        self.stdout_handler.addFilter(Allow_Logger_By_Name(self.log.name))
        if self.stdout_handler not in self.log.handlers:
            self.log.addHandler(self.stdout_handler)
        else:
            print "Not adding stdout handler for this eulogger:" +str(self.identifier)

        #Now add the file handlers...
        if use_global_log_files:
            self.file_info_list = self.get_parent_logger_files()
        if (self.logfile):
            self.file_info_list.append(File_Handler_Info(self.logfile,self.logfile_level))
            #If the clear flag is set remove the file first...
            if (self.clear_file):
                try:
                    os.remove(self.logfile)
                except Exception, e:
                    print "Error while attempting to remove log file '" + self.logfile + "', err:" + str(e)
            if make_log_file_global:
                self.add_muted_file_handler_to_parent_logger(self.logfile,self.logfile_level)
        for fileinfo in self.file_info_list:
            file_hdlr = logging.FileHandler(fileinfo.filepath)
            file_hdlr.setFormatter(self.file_format)
            file_hdlr.setLevel(fileinfo.level)
            #Add filter so only log records from this child logger are handled
            file_hdlr.addFilter(Allow_Logger_By_Name(self.log.name))
            #Make sure this is not a duplicate handler or this file is a dup of another handler
            if file_hdlr not in self.log.handlers:
                add = True
                for h in self.log.handlers:
                    if h.stream.name == file_hdlr.stream.name:
                        add = False
                        self.log.debug('File already has log handler:' + str(logfile.filepath))
                        break
                if add:
                    self.log.addHandler(file_hdlr)
            else:
                print "Not adding logfile handler for this eulogger:" +str(self.identifier)

        self.log.debug(str(self.identifier) + ": Eulogger init test message. Init complete")

    def add_muted_file_handler_to_parent_logger(self,filepath, level):
        file_handler = logging.FileHandler(filepath)
        file_handler.setLevel(level)
        file_handler.addFilter(Mute_Filter())

    def get_parent_logger_files(self):
        files = []
        for h in self.parent_logger.handlers:
            if isinstance(h, logging.FileHandler):
                files.append(File_Handler_Info(h.stream.name, h.level))
        return files

    def getChild(self, logger, suffix):
        """
        ## Add this for 2.6 support, this was implemented in 2.7...###
        """
        if hasattr(logger,'getChild'):
            return logger.getChild(suffix)
        else:
            if logger.root is not logger:
                suffix = '.'.join((logger.name, suffix))
            return logger.manager.getLogger(suffix)


class File_Handler_Info():
    def __init__(self, filepath, level):
        if not filepath or not level:
            raise Exception("File_Handler_Info None option not allowed, filepath:"+str(filepath)+",level:"+str(level))
        self.filepath = filepath
        self.level = level

class Allow_Logger_By_Name(logging.Filter):
    """
    Only messages from this logger are allow through to prevent duplicates from other loggers of same level, etc..
    """
    def __init__(self, name=""):
        logging.Filter.__init__(self, name)

    def filter(self, record):
        return record.name == self.name

class Mute_Filter(logging.Filter):
    def filter(self, record):
        return False



 
